/**************************************************************************/ 
/*                                                                        */ 
/*          Copyright (c) 2002 by Control Technology Corporation          */ 
/*                          All Rights Reserved                           */ 
/*                                                                        */ 
/*  This software is copyrighted by and is the sole property of Control   */ 
/*  Technology Corporation.  All rights, title, ownership, or other       */ 
/*  interests in the software remain the property of Control Technology   */ 
/*  Corporation.  Any unauthorized use, duplication, transmission         */ 
/*  distribution, or disclosure of this software is expressly forbidden.  */ 
/*                                                                        */ 
/*  This Copyright notice may not be removed or modified without prior    */ 
/*  written consent of Control Technology Corporation.                    */ 
/*                                                                        */ 
/*  Control Technology Corporation reserves the right to modify this      */ 
/*  software without notice.                                              */ 
/*                                                                        */ 
/*  Control Technology Corporation           info@ctc-control.com         */
/*  25 South Street                          http://www.ctc-control.com   */
/*  Hopkinton, MA  02038                                                  */
/*                                                                        */
/**************************************************************************/
/* MODULE: CoreFunc.h                                                     */
/*                                                                        */ 
/* MODULE DESCRIPTION:                                                    */
/*                                                                        */ 
/* Base include module required for all User Function files.  Defines and */
/* includes any other Quickstep specific include files.  Such as          */
/* 1.  Protype declaration of Quickstep OS function calls which first     */
/*     verify table existance prior to allowing access.                   */
/* 2.  tx_api.h - Thread X include files                                  */
/* 3.  Errors.h - Quickstep OS RETVAL function return value definitions   */
/* 4.  logcodes.h - Debug specific logging codes                          */
/* 5.  uTable.h - Virtual function table definitions                      */  
/* 6.  MALLOC_DATA_SIZE - Memory Heap size for dynamic allocation         */
/*     (malloc/free)                                                      */
/* 7.  coreTable - Base Quickstep OS virtual table global references      */
/* 8.  printf - definition for redirection to UDP debugger                */      
/*                                                                        */ 
/* REVISION HISTORY:                                                      */
/*                                                                        */ 
/**************************************************************************/
#include <stdarg.h>

#include "Errors.h"

#ifndef	__CORE_FUNCTIONS_H
#define	__CORE_FUNCTIONS_H
#ifdef   __cplusplus
extern   "C" {
#endif

/* Include the log codes, making sure we are not defining the table */
#ifdef APP_EVENT_TABLE_CREATE
#undef APP_EVENT_TABLE_CREATE
#endif
#define APP_EVENT_TABLE_CREATE 0
#include "tx_api.h"

#ifndef NULL
#define NULL 0
#endif

#ifndef	TRUE
#define	TRUE	1
#endif

#ifndef	FALSE
#define	FALSE	0
#endif

#ifndef	CRLF
#define	CRLF	"\r\n"
#endif

// Amount of heap memory to reserve for malloc calls
#define MALLOC_DATA_SIZE		(0x80000)

#define BIT0				0x00000001
#define BIT1				0x00000002
#define BIT2				0x00000004
#define BIT3				0x00000008
#define BIT4				0x00000010
#define BIT5				0x00000020
#define BIT6				0x00000040
#define BIT7				0x00000080
#define BIT8				0x00000100
#define BIT9				0x00000200
#define BIT10				0x00000400
#define BIT11				0x00000800
#define BIT12				0x00001000
#define BIT13				0x00002000
#define BIT14				0x00004000
#define BIT15				0x00008000
#define BIT16				0x00010000
#define BIT17				0x00020000
#define BIT18				0x00040000
#define BIT19				0x00080000
#define BIT20				0x00100000
#define BIT21				0x00200000
#define BIT22				0x00400000
#define BIT23				0x00800000
#define BIT24				0x01000000
#define BIT25				0x02000000
#define BIT26				0x04000000
#define BIT27				0x08000000
#define BIT28				0x10000000
#define BIT29				0x20000000
#define BIT30				0x40000000
#define BIT31				0x80000000

typedef unsigned char   UINT8;
typedef unsigned short  UINT16;
typedef unsigned long   UINT32;
typedef signed char     INT8;
typedef signed short    INT16;
typedef signed long     INT32;

typedef unsigned long   STDVAL;
typedef unsigned int    INDX;
typedef UINT8	QCODE;

#define STEP_SIZE 2
#define UNKNOWN_STEP 0xFFFF
typedef UINT16 STEPNUM;
#define CONFIG_MAX_TASKS               96   /* number of TASKs */
#define CONFIG_MAX_STEPNUM           4096   /* highest step number */ 
#define MAX_TASKS_8  ((CONFIG_MAX_TASKS + 7)/8)

typedef enum {START_INIT = 0, /*system is performing start-up initialization*/
              RESET,          /*system is resetting*/
              AT_RESET,       /*system reset complete*/
              RESET_INPUT,    /*system reset input detected*/
              RESETSTOP_INP,  /*system reset input and stop input/command detected*/
              RESTART,        /*system is resetting and will "restart" to running*/
              STOPPED,        /*system stopped*/
              PROGRAMMING,    /*system in programming mode*/
              RUNNING,        /*system is running*/
              FAULT,          /*system running has encountered a fault*/
              FAULT_STOPPED,  /*system stopped after encountering a fault*/
              SHUTDOWN,        /*system commanded to shutdown*/
			  OLDSTATEEMPTY   /* used by safe state logic to know when can save current state */
              } SYSMODE;

typedef enum{ TASK_UNUSED=0,  /* task is not in use */
              TASK_DONE,      /* task is complete */
              TASK_ERROR,     /* error occurred, task halted */
              TASK_PAUSED,    /* task is paused (used for single-step) */
              TASK_STOPPED,   /* task stopped */
              TASK_CREATED,   /* new task created this cycle */
              TASK_START,     /* task is started/restarted at first step */
              TASK_GOTOSTEP,  /* task is branching to new step */
              TASK_NEWSTEP,   /* task is running first pass of step */
              TASK_RUNSTEP,    /* task is running step */
              TASK_SS_START,  /* task is started/restarted at 1st step, single step active */
              TASK_SS_STEP,   /* tasks runs first pass of step, single step active */
              TASK_SS_RUN     /* task is running step, single step active */
              } TASK_STATE;

// Breakpoint type enabled
#define BREAK_NONE 0
#define BREAK_READ 0x0001
#define BREAK_WRITE 0x0002
#define BREAK_READWRITE 0x0003
#define BREAK_STEP 0x0008
#define MAX_BREAKSTRING 110
typedef struct
{
	UINT8 breakActive;      // 0 = idle, 1 = triggered
	UINT8 breakType;		// OR of bits 
	UINT8 breakCause;		// if breakActive, reason break occured
	short breakStep;		// step to break on, 0 if none
	short breakRegister;	// register to break on, 0 if none
	UINT8 qualifier;		// == 0, > 1, >= 2, < 3, <= 4
	int   qualData;
	char breakString[MAX_BREAKSTRING+1];	// string of what breakpoint is set to [S1000 R12000 READ > 500000]
}BREAKPOINT;

#define BREAK_DISABLED 0
#define BREAK_ENABLED 1
#define BREAK_TRIGGERED 2

#define BREAK_QUALIFIER_EQUAL 1
#define BREAK_QUALIFIER_GREATER 2
#define BREAK_QUALIFIER_GREATER_EQUAL 3
#define BREAK_QUALIFIER_LESSTHAN 4
#define BREAK_QUALIFIER_LESSTHAN_EQUAL 5

#define MAX_BREAKPOINTS	4
typedef struct
{
   UINT8	  singleStep; // if 1 then in single step mode
   UINT8	  breakTriggered;
   int		  instructExecuting;
   BREAKPOINT breakpoints[MAX_BREAKPOINTS];
   UINT8	  oldState;   // state prior to breakpoint
   void 	  *handle;	  // telnet session to send results back on   
}BREAKINFO;	

#define PEER_REG_BASE 40000				//  All registers greater than or = this are remote (assumes we are one less than asked for)
#define TASK_PEER_DISABLED 0x0000		// PEER access not used.
#define TASK_PEER_READ 0x0001			// enable remote on read operation
#define TASK_PEER_WRITE 0x0002			// enable remote on write operation
#define TASK_PEER_OPERAND1 0x0004		// enable remote on OPERAND1 access
#define TASK_PEER_OPERAND2 0x0008		// enable remote on OPERAND2 access
#define TASK_PEER_STORE 0x0100			// enable remote store instruction
#define TASK_PEER_IF 0x0200				// enable remote if instruction
#define TASK_PEER_MONITOR 0x0400		// enable remote monitor instruction
#define TASK_PEER_DELAY 0x0800			// enable remote delay instruction
#define TASK_PEER_FLAG 0x1000			// Enable remote flag instruction
#define TASK_PEER_PERSITANCE 0x80000000	// Step does not cancel remote access

typedef struct tcb {
               QCODE      *step_ptr;  /* pointer to start of current step */
               STEPNUM    step_1st;   /* number of 1st step task executed */
               STEPNUM    step_num;   /* current step number */
			   STEPNUM    step_next;  /* next step number */ 
               UINT8      repeat;     /* repeat count of DO */
               TASK_STATE state;      /* task state */
               RETVAL     status;     /* operation status */
			   short	  stepLockFlag;  // If set will not task switch when enter next 
			   							// step, stays with this task
			   RETVAL     stepC_APIBranch;	// if this is nonzero then 'taskStepBranch' was called
			   							    // and this is the return status to use after executing
			   							    // a step, typically, END_OF_STEP for branch to occur
			   							    // but could be a fault condition. 
			   short	  peeridx;	  /* peer.ini index */

			   // BIT0 - read
			   // BIT1 - write
			   // BIT2 - Operand 1
			   // BIT3 - Operand 2
			   // BIT4 - Store inst.
			   // BIT5 - IF instruct.
			   // BIT6 - MONITOR instruct.
			   // BIT7  - FLAG instruct.
			   // BIT32 - PERSISTENT across steps
			   int	  	  peerrules;
			   int        activeAccessRules;  /* rules for instruction currently executing */
			   RETVAL     peerstatus;
			   BREAKINFO  *breakInfo;
			   void		  *localRegs;   	// local registers
 			   void * 	  qs4TaskLocal;		// Reserved QS4 storage
			   void *     userCStorage;     // General userCStorage
			   short	  taskNum;			// Task number, base 0
			   unsigned int taskHandle;		// every task has a unique task handle, assigned upon creation
              UINT8      wait_for[MAX_TASKS_8];
                                      /* Each bit represents an offspring */
               } TASK;

typedef struct ts {
               STEPNUM    step_num;
               TASK_STATE state;
               UINT8      wait_for[MAX_TASKS_8];
               } TASK_STATUS;

typedef struct

{
	TASK *task;	// current TASK *, or NULL if not a task (CTCMON, etc)
	void *RegVal;	// same as what is passed in regVRead and regVWrite
	int type;	// same as what is passed in regVRead and regVWrite
	void *vptr;  	// Actual current Variant structure pointer for read operations

} REGVINFO;

#include "logcodes.h"	// UDP Debugger log type definitions
#include "uTable.h"		// Virtual table definitions
#include "variant.h"	// Variant register definitions

/***************** QS4 Timer Definitions *******************/
#define peripheral_reg  volatile unsigned long
#define AT91C_BASE_TCB0           (0xFFFA0000L)         // Timer/Counter Block 0 Base Address
#define     AT91C_BASE_TC0    (AT91C_BASE_TCB0 + 0x00)  // Timer/Counter 0 Base Address
#define OFFSET_TC_CV            (0x10) // Counter Value
#define AT91C_TC0_CV         (*((peripheral_reg *)(AT91C_BASE_TC0 + OFFSET_TC_CV)))

/**************** QS4 Control Definitions ******************/

// Reason Codes
#define QS4C_BOARD_ANALOG_READ 1	// Analog board just scanned and ain_value data updated
#define QS4C_ALL_ANALOG_READ 2		// All analog boards scanned in the system, loop complete

// Reason 'info' structure passed
// Board structure reference for Analog inputs, used for callback
#define NUM_MAX_AINS 32
typedef struct
{
	void *SYSTEMinfo;			// system information structure
	unsigned short slot_num;	// board slot in system, 0 based
	unsigned short type;
	unsigned char *slot_addr;	// pointer to board dual ported memory
	unsigned short pca;		// board build option
	short num_ain;			// number of analog inputs in ain_value array, 0 to NUM_MAX_AINS
	short num_aout;			// number of analog outputs on board
	short ain_start;		// 0 to ...  first AIN number in Quickstep sequence on this card
	int ain_value[NUM_MAX_AINS];
} SlotMem_AIN;

//--------------------------------------

extern int initCoreFunction(void);		// This function must be called during main() to init Quickstep OS table pointer
extern const VTABLE_QS *coreTable;      // Table point to get initialized at the start of exection

// printf declaration to redirect output to Quickstep OS UDP debugger
// DO NOT CHANGE or use the standard library printf statement!
#define printf	if (coreTable) (*coreTable->printf)

/********* Register Access ***************/
RETVAL regRead(UINT16 RegNum, INT32 *RegVal);
RETVAL regWrite(UINT16 RegNum, INT32 RegVal);
// Variant register access
RETVAL regVRead( void *task, UINT16 RegNum, void *RegVal, int type);
RETVAL regVWrite( void *task, UINT16 RegNum, void *RegVal, int type);

/********* Task Control ******************/
RETVAL taskSetBranch( void *task, int step);
void taskStepLock(void *task);
TASK* createTask(int stepnumber);
RETVAL killTask(TASK*);
TASK *getTaskFromHandle(unsigned int handle);

/********* Communications ****************/
	/* This function sends a message (undefined format) of length "size" out the specified "port" using the 
       appropriate driver. 
    */
RETVAL commSendMsg( INDX port, UINT8 *msg, UINT16 size);
	/* This function sends a generic message command (undefined format) of length "size" to the 
       specified "port" using the appropriate driver. Any response is copied back to "msg"; the size of the
       of the response is copies back to "size"
    */
RETVAL commGenericCmd( INDX port, UINT8 *msg, UINT16 *size);
    /* This function invokes the driver associated with "port" and returns a pointer to the buffer that contains the 
       request message.  The message contents are NOT copied.  If a request message from the port is not available, the
       "buf" pointer is set to NULL.  "size" will have the number of bytes in the buffer stored in it, this is not
       necessarily the same as the number of bytes in the message.
    */
RETVAL commReadMsg( INDX port, UINT8 **buf, UINT16 *size);
int commNetworkOpen( USERCONNECTION *user);
void commNetworkClose( USERCONNECTION *user);
int commNetworkSend( USERCONNECTION *user);

/************ Motion Routines ****************/
UINT16 Motion_GetNumMotors(void);
RETVAL Motion_Get_Attribute( UINT8 MotorNum, UINT8 Attribute, STDVAL *value);
RETVAL Motion_Put_Attribute( UINT8 MotorNum, UINT8 Attribute, STDVAL value);
RETVAL Motion_Ready( UINT8 MotorNum, STDVAL *value);
RETVAL Motion_Simple_Command( UINT8 MotorNum, UINT8 Command);

RETVAL RPC_axis_param_read(TASK *task, int axis, int parameter, int type, void *value, int *rpc_result, int peeridx);
RETVAL RPC_axis_param_write(TASK *task, int axis, int parameter, int type, void *value, int *rpc_result, int peeridx);
RETVAL RPC_clear_all_MSB(TASK *task, int axis, int *rpc_result, int peeridx);
RETVAL RPC_axis_start_MSB(TASK *task, int axis, int msb_number, int *rpc_result, int peeridx);
RETVAL RPC_axis_stop_MSB(TASK *task, int axis, int msb_number, int *rpc_result, int peeridx);
RETVAL RPC_axis_status(TASK *task, int axis, int *status, int *rpc_result, int peeridx);
RETVAL RPC_load_MSB(TASK *task, int axis, int msb_number, void* msb, int size, int *rpc_result, int peeridx);

/*********** System Functions ***************/
unsigned long systemTics(void);		// number of timer tics since powerup, 1ms/tic currently
void *addResourceFilter(RESOURCE_INFO *rsc, FILTER_FUNCTION func);	// add a resource filter function
RETVAL removeResourceFilter(void *handle);		// remove a resource filter function
RETVAL fireWatchdog(void);		// watchdog timer must be reset every 85ms else call this

/*********** Base Threading Functions (Advanced users only) ************/
unsigned int _tx_thread_create( TX_THREAD *thread_ptr,       /* Task Structure */
						/* Task Name */
                        char *name_ptr,
						/* Routine and Parameter to pass */
                        void (*entry_function)(unsigned long), unsigned long entry_input,
                        /* Stack start and length */
                        void *stack_start, unsigned long stack_size,
                        /* Priority and Threshold */
                        unsigned int priority, unsigned int preempt_threshold, 
						/* time slice */
                        unsigned long time_slice, unsigned int auto_start);

unsigned int _tx_thread_suspend(TX_THREAD *thread_ptr);  // suspend a thread
unsigned int _tx_thread_resume(TX_THREAD *thread_ptr);   // resume a suspended thread
unsigned int _tx_thread_sleep(unsigned long tics);	// amount of time to sleep, only call from a thread
unsigned int _tx_mutex_create(TX_MUTEX *mutex_ptr, char *name,unsigned int inherit);  // create a mutex
unsigned int _tx_mutex_get(TX_MUTEX *mutex_ptr, unsigned long wait_option);  // get control of a mutex
unsigned int _tx_mutex_put(TX_MUTEX *mutex_ptr);	// give up control of a mutex
unsigned int _tx_mutex_delete(TX_MUTEX *mutex_ptr);	// delete a mutex

/*********** Script Execution Environment commands ************/
CMDPARSER *CTC_allocateCommandParser(int maxbuffer);
void CTC_releaseCommandParser(CMDPARSER *pCmdP);
long CTC_runCommandParser(CMDPARSER *pCmdP, char *cmd);
int CTC_getCommandResponse(CMDPARSER *pCmdP,char *buf);

// diagnostics
RETVAL logEvent(EventCode event, long parameter);
RETVAL logEventPlus(EventCode event, long firstParameter, long secondParameter);
int sprintf(char *buffer, const char *format, ...);
int vsprintf(char *buffer, const char *format, va_list ap);
// return amount of memory available
long mem_avail(void);
// return amount of memory used
long mem_used(void);

// memory allocation for malloc support
unsigned int _tx_byte_allocate(TX_BYTE_POOL *pool_ptr,
	void **memory_ptr,unsigned long memory_size,unsigned long wait_option); // give malloc memory back
unsigned int _tx_byte_release(void *memory_ptr); // give malloc memory back
unsigned int _tx_byte_pool_create(TX_BYTE_POOL *pool_ptr,
	char *name_ptr, void *pool_start, unsigned long pool_size);
unsigned int _tx_byte_pool_delete(TX_BYTE_POOL *pool_ptr);
unsigned int _txe_byte_pool_delete(TX_BYTE_POOL *pool_ptr);
// interrupt control
//unsigned int disableInts(unsigned int);			// turn interrupts off
//unsigned int enableInts(unsigned int);			// turn interrupts on

// Step control
RETVAL enterStepAtomicity();
RETVAL exitStepAtomicity();

// File system access
// Open the desired file to type of access desired and return handle for further access
CTC_FILE CTC_fopen(char *filename, char *mode);

// Get file length
unsigned long CTC_filelength(CTC_FILE apFile);

// Close file openned by CTC_fclose()
int CTC_fclose(CTC_FILE apFile);

// Remove our directory reference so thread can be re-used, recommend that CTC_fclose 
// still be called on each handle else will have a memory leak.
void CTC_fcloseall();

// Check for end of file
int CTC_feof( CTC_FILE apFile );

// Write to file
unsigned int CTC_fwrite( void *buffer, unsigned int size, unsigned int count, CTC_FILE apFile );

// Get current file position
long CTC_ftell(CTC_FILE apFile);

// Flush open file
void CTC_fflush(CTC_FILE apFile);

// Read from file
unsigned int CTC_fread(void *apByteBuffer, unsigned int size, unsigned int length, CTC_FILE apFile );

// Get a line terminated with CR LF from an open file
char *CTC_fgets(char *string, int count,CTC_FILE handle);

// Write a null terminated string to an open file
int CTC_fputs( char *buffer, CTC_FILE apFile);

// Seek to specific position in a file.  Origin is ignored, always from start of file
int CTC_fseek( CTC_FILE handle, long offset, int origin );

// Return pointer to the name of the openned file contained within a directory block
char *CTC_getFileName(CTC_FILE handle);

// DIRECTORY FUNCTIONS

// Delete a directory
int CTC_rmdir(char *DirName);

// Get current position in a directory tree
int CTC_pwd(char *DirName);

// Change directories
int CTC_cd(char *DirName);

// Create a new directory
int CTC_mkdir(char *DirName);

// Format the disk
int CTC_formatDisk();

// Get full file name with path information
int CTC_getFullFileName(CTC_FILE handle, char *name);

// Get Flash disk information at disk level
void CTC_volumeGetInfo(unsigned long *TotalSize, unsigned long *TotalAvail, unsigned long *TotalDeleted);

// Initialize CTC_FIND_DATA structure and get information on first file, similiar to MSDOS findfirst()
int CTC_findfirst( char *lpFileName,CTC_FIND_DATA *lpFindFileData);

// Get CTC_FIND_DATA information on next file in directory tree, similiar to MSDOS findnext()
int CTC_findnext( CTC_FIND_DATA *lpFindFileData); 

// Terminate the directory tree walk initiated with the CTC_findfirst() call
void CTC_findclose(CTC_FIND_DATA *lpFindFileData);

// Delete a file
int CTC_remove(char *filename);

// Rename a file (ram disk only)
int CTC_rename(char *fromName,char *toName);

// Rename a directory (ram disk only)
int CTC_rendir(char *fromName,char *toName);

// Get information on a file that is not open, by name and path only
int CTC_getFileInfo(char *Name, CTC_FIND_DATA *lpFindFileData);


/* Communications Command Requests */
#define COMMCMD_CMND_RESET   0
#define COMMCMD_GENERIC     12
#define COMMCMD_RQST_CTCBIN   1
#define COMMCMD_RQST_MSG      2
#define COMMCMD_RQST_QUERY    3
#define COMMCMD_RQST_GETCH    4
#define COMMCMD_RQST_GETNUM   5
#define COMMCMD_RQST_GETCNT   6
#define COMMCMD_RQST_CLRBUF   7
#define COMMCMD_RQST_NEWBAUD  8
#define COMMCMD_RQST_PARSING  9
#define COMMCMD_RQST_DEVCNET 10
#define COMMCMD_RQST_VERSION 11
#define COMMCMD_RQST_SET_MODBUS 13

/* Communications Command Responses */
#define COMMCMD_RSPN_OK       0
#define COMMCMD_RSPN_BUSY     1
#define COMMCMD_RSPN_READY    2
#define COMMCMD_RSPN_ACK     64
#define COMMCMD_RSPN_NACK    65

#define COM1 1
#define COM2 2

// Script mask errors
#define SCRIPT_ERRMASK_FATAL 0x00800000
#define SCRIPT_ERRMASK_SYNTAX 0x00400000		// Syntax error
#define SCRIPT_ERRMASK_FILE 0x00200000		// general file failures... format, etc...
#define SCRIPT_ERRMASK_FTP_CONNECT 0x00100000  // FTP Connection attempt failure, both data and control socket
#define SCRIPT_ERRMASK_FTP_LOSTCONNECT 0x00080000  // Lost connection with host error
#define SCRIPT_ERRMASK_FTP_COMMAND 0x00040000	// general FTP Command failure
#define SCRIPT_ERRMASK_FTP_NOTFOUND 0x00020000	// Operation failed, file or directory not exist on host
#define SCRIPT_ERRMASK_FTP_SECURITY 0x00010000	// Password failed

#ifdef   __cplusplus
        }
#endif

#endif	//	__CORE_FUNCTIONS_H
