/**************************************************************************/ 
/*                                                                        */ 
/*          Copyright (c) 2002 by Control Technology Corporation          */ 
/*                          All Rights Reserved                           */ 
/*                                                                        */ 
/*  This software is copyrighted by and is the sole property of Control   */ 
/*  Technology Corporation.  All rights, title, ownership, or other       */ 
/*  interests in the software remain the property of Control Technology   */ 
/*  Corporation.  Any unauthorized use, duplication, transmission         */ 
/*  distribution, or disclosure of this software is expressly forbidden.  */ 
/*                                                                        */ 
/*  This Copyright notice may not be removed or modified without prior    */ 
/*  written consent of Control Technology Corporation.                    */ 
/*                                                                        */ 
/*  Control Technology Corporation reserves the right to modify this      */ 
/*  software without notice.                                              */ 
/*                                                                        */ 
/*  Control Technology Corporation           info@ctc-control.com         */
/*  25 South Street                          http://www.ctc-control.com   */
/*  Hopkinton, MA  02038                                                  */
/*                                                                        */
/**************************************************************************/
/* MODULE: Errors.h                                                       */
/*                                                                        */ 
/* MODULE DESCRIPTION:                                                    */
/*                                                                        */ 
/* Definition of error codes returned by functions with RETVAL.  These    */
/* functions are the same as those called by Quickstep itself.            */
/*                                                                        */ 
/* REVISION HISTORY:                                                      */
/*                                                                        */ 
/**************************************************************************/

#ifndef RET_VALUE_H
#define RET_VALUE_H yes

/* -----------------------------------------------------------------------*/
/*		Return Values                                                     */
/* -----------------------------------------------------------------------*/

typedef enum  {
        SUCCESS                 = 0,
        ERROR_ILLEGAL_INST      = 1,  /* illegal instruction opcode */
        ERROR_CORRUPT           = 2,  /* corrupt program data */
        ERROR_NO_STEP           = 3,  /* dest. step doesnt exist */
        ERROR_BAD_THUMB         = 4,  /* bad data from thumbwheel */
        ERROR_NO_STEP_1         = 5,  /* step#1 doesnt exist */
        ERROR_MAX_TASKS         = 6,  /* too many tasks */
        ERROR_NO_STEPPER_MOTOR	= 7,  /* no such stepping motor # */
        ERROR_STEPPER_NOT_READY         = 8,  /* stepping motor not ready */
        ERROR_NOT_PROF          = 9,  /* stepping motor not profiled*/
        ERROR_NO_SERVO          = 10, /* no such servo # */
        ERROR_SERVO_NOT_READY       = 11, /* servo not ready */
        ERROR_SERVO             = 12, /* servo error, n.e.c. */
        ERROR_NO_REG            = 13, /* no such register exists */
        ERROR_NO_COL            = 14, /* no such data table col */
        ERROR_NO_ROW            = 15, /* no such data table row */
        ERROR_NO_PROTO          = 16, /* no such prototype board */
        ERROR_SAMPLE_T          = 17, /* illegal sample time */
        ERROR_NO_AIN            = 18, /* no such analog input # */
        ERROR_NO_AOUT           = 19, /* no such analog output # */
        ERROR_NO_DISP           = 20, /* no such display # */
        ERROR_DIG_INP           = 21, /* no such digital input # */
        ERROR_DIG_OUT           = 22, /* no such digital output # */
        ERROR_NO_THUMB          = 23, /* no such thumbwheel # */
        ERROR_ILLEGAL_VAL       = 24, /* illegal (data table) value */
        ERROR_XMIT              = 25, /* transmitter busy */
        ERROR_DIV_ZERO          = 26, /* divide-by-zero error */
        ERROR_OO_RANGE          = 27, /* generic data-out-of-range */
        ERROR_HARDWARE          = 28, /* hardware fault */
        ERROR_NET_ACCESS        = 29, /* network access problem */
        ERROR_NET_TIMEOUT       = 30, /* network access timeout */
        ERROR_NET_BUSY          = 31, /* network access busy */
        ERROR_NET_RQST_LOST     = 32, /* network request lost */
        ERROR_NET_BAD_RESPONSE  = 33, /* network bad response */
        ERROR_NET_BAD_MSG       = 34, /* network bad return message */
        ERROR_NO_COMM_PORT      = 35, /* no such communications port */
        ERROR_COMM_PROTOCOL     = 36, /* error in request/reply handshake */
        ERROR_NO_FLAG           = 37, /* bad flag number selected */
        ERROR_NO_DELAYTIMER     = 38, /* bad delaytimer selected */
        ERROR_OUT_OF_SOFTCNTRS  = 39, /* Out-of-softcounters */
        ERROR_AIN_SCALING       = 40, /* Error in fetching or calculating analog scaling */
        ERROR_NOT_CALIBRATED    = 41, /* Analog stick not calibrated, later servo too */
		ERROR_ATMEL_ISP			= 42, /* problem programming Atmel chip	 */
		ERROR_FILE_OPEN			= 43, /* error trying to open file requested, not exist? */
		ERROR_FILE_FGETS      	= 44, /* fgets returned null pointer, not found */
		ERROR_MALLOC			= 45, /* malloc failed */
		ERROR_ANALOG_NOT_READY 	= 46, /* Analog stick not responding*/
		ERROR_AOUT_SCALING      = 47, /* Error in fetching or calculating analog scaling */
        ERROR_INVALID_PCA    	= 48, /* Illigal build of an Atmel board. */
		ERROR_VIRTUALIO_CONNECTION = 49,  /* Lost connection with Virtual Controller */
        ERROR_TASK = 50,
        ERROR_TASK_STATUS = 51,
        ERROR_DELAY_REJECTED = 52,         /* Time delay not accepted
		                                 not necessarily an error */
        ERROR_IOACCESS = 53,               /* Error Accessing Hardware I/O */
        ERROR_IODRIVER = 54,               /* Generic Error in IODRIVER */
        INVALID_PARAMETER = 55,            /* Invalid parameter was passed, should never occur */
        ERROR_EXT_DATA_DESC = 56,          /* Invalid extended data descriptor */
        ERROR_SPI_OVERRUN = 57,
        ERROR_SPI_TIMEOUT = 58,
		ALTERNATE_ERROR_ILLEGAL_INST = 59,	// use this instead of 1 since success can be 1 or 0 depending where used
		ERROR_FTP_DISCONNECT = 60,			// attempted to connect on something already connected
		ERROR_FTP_CONNECT = 61,				// attempted FTP Client operation but not connected
		ERROR_FTP_COMMAND = 62,			// General FTP command failed
		ERROR_LOST_EXPANSION = 63,			// Lost Expansion during operation
		ERROR_NOT_SET = 64, 		// Used for dual port to set error to not defined yet
		ERROR_IO_DUALPORT_UNDEFINED = 65,   // Dualport device on IO bus ID seen but cpu not running...
 		ERROR_NOT_DEFINED = 66,			  /* Virtual table function call not defined */

		// Internal below
        END_OF_STEP = 67,                  /* Code to indicate "End of Step" was found,
                                         not an error. */
        END_ALL_TASKS                 /* Code to indicate last task has 
                                         successfully completed. */
} RETVAL /* Return Value */;

#endif /* #ifndef RET_VALUE_H */
/* END OF DOCUMENT */

