/*
***************************************************************
Event Logging
***************************************************************
*/


/* 
The first set of macros simply defines the constants for the 
various groups of codes it is used by all modules that may
include this header file and therefore is protected from 
multiple loadings, even though some compilers don't complain 
about benign redefinitions (identical #define statements)
*/

#ifndef   _Event_logging_CODES_H_
#define   _Event_logging_CODES_H_

/* make sure that the header is processed correctly in a C++ module. */
#ifdef   __cplusplus
extern   "C" {
#endif

// Base for log codes in each module
#define LOG_5102INTR_C_BASE 1700
#define LOG_SCRIPTS_CPP_BASE 1800
#define LOG_PEER_CPP_BASE 1900
#define LOG_VIRUALIO_CPP_BASE 2000
#define LOG_QSMAIN_C_BASE 2100
#define LOG_NETWORK_CPP_BASE 2200
#define LOG_SNTP_CPP_BASE 2300
#define LOG_TCPSERIAL_CPP_BASE 2400
#define LOG_OSAPI_CPP_BASE 2500
#define LOG_DISKSYS_CPP_BASE 2600
#define LOG_QS51XX_C_BASE 2700
#define LOG_5100COMM_C_BASE 2800
#define LOG_5100HW_C_BASE 2900
#define LOG_5100SLOT_C_BASE 3000
#define LOG_DISKMGR_CPP_BASE 3100
#define LOG_USERFUNCTIONS_CPP_BASE 4000

#define LOG_CTC_SOCKETS_BASE 4100
#define LOG_CTC_THREADS_BASE 4200
#define LOG_CTC_MUTEXS_BASE 4300

#define LOG_TCP_APPS_BASE 4500

#define   LogFlagMask       0xffff8000L
#define   LogCodeMask       0x00000fffL
#define   LogNameParameter  0x00001000L   /* First Parameter points to a Name */
#define   Log2Parameters    0x00004000L   /* Two parameters are logged */
#define   LogError          0x00008000L   /* event is an error */

#define   LogHardware       0x00010000L
#define   LogOScalls        0x00020000L
#define   LogTimers         0x00040000L
#define   LogIoEvents       0x00080000L
#define   LogCAN            0x00100000L
#define   LogEthernet       0x00100000L
#define   LogServo          0x00200000L
#define   LogCommStack      0x00400000L
#define   LogMessages       0x00400000L
#define   LogDeviceNet      0x00800000L
#define   LogProfile        0x01000000L
#define   LogRegistration   0x02000000L
#define   LogCTClink        0x04000000L
#define   LogScanner        0x08000000L
#define   LogApplication    0x10000000L

#define   LogAlways         0xffff8000L

#define   ForceLog          LogAlways
#define   LogErrors         LogError


/* Codes not used B,F,G,J,K,L,V,W,X,Y,Z*/
#define LH  | LogHardware
#define LO  | LogOScalls
#define LT  | LogTimers
#define LI  | LogIoEvents
#define LC  | LogCAN
#define LN  | LogEthernet
#define LP  | LogProfile
#define LM  | LogMessages
#define LQ  | LogCommStack
#define LR  | LogRegistration
#define LS  | LogServo
#define LD  | LogDeviceNet
#define LL  | LogCTClink
#define LU  | LogScanner
#define LA  | LogApplication
#define LE  | LogError
#define L   | LogAlways
#define NAM | LogNameParameter
#define L2  | Log2Parameters

#endif    // _Event_logging_CODES_H_

#ifndef APP_EVENT		// this is weird code since in eventprn we include to init a structure...
#define APP_EVENT
#if     APP_EVENT_TABLE_CREATE == 0
#define eventDefine(n, v, s)  n = v,
#define eventDefStart  typedef enum {
#define eventDefEnd    } EventCode;
#else
#define eventDefine(n, v, s)  {n , s},
#define eventDefStart {  
#define eventDefEnd   };    
#endif

eventDefStart
    eventDefine(EVENT_HW_FAIL                  , (0x001 LH LE)      , "Hardware Failed, code")
    eventDefine(EVENT_INTERRUPT                , (0x002 LH L2 LE)   , "Unvectored Interrupt")
    eventDefine(EVENT_RX_OVERRUN               , (0x003 LH LE)      , "Receive Overrun, channel")
    eventDefine(EVENT_TX_OVERRUN               , (0x004 LH LE)      , "Transmit Overrun, channel")
    eventDefine(EVENT_CAN_HW_FAIL              , (0x005 LH LE)      , "CAN Hardware Failed, code")

    eventDefine(EVENT_FASTLOG                  , (0x008 LE)         , "Fast Log time")
    eventDefine(EVENT_MEMORY                   , (0x009 LE L2)      , "Memory Allocation Failure")
    eventDefine(EVENT_CONFIG                   , (0x00a LE)         , "Invalid Configuration")
    eventDefine(EVENT_SHUTDOWN                 , (0x00b LE)         , "Shutdown Error")

    eventDefine(EVENT_INIT_LOG                 , (0x010 LH LE)      , "Clear Log")
    eventDefine(EVENT_LIB_SYSCALL              , (0x018 LE L2)      , "GNU SysCall")

    eventDefine(EVENT_SLOT_INIT                , (0x020 LH L2)      , "Init. Slots")
    eventDefine(EVENT_SLOT_SELECT              , (0x021 LH)         , "Sel. Slot")
    eventDefine(EVENT_SLOT_WRITE               , (0x022 LH L2)      , "Slot Write")
    eventDefine(EVENT_SLOT_READ                , (0x023 LH L2)      , "Slot Read")
    eventDefine(EVENT_SLOT_READ_ID             , (0x024 LH)         , "Slot ID")

    eventDefine(EVENT_SLOT_SPI_SELECT          , (0x028 LH)         , "SPI Sel.")
    eventDefine(EVENT_SLOT_SPI_TX              , (0x029 LH LE)         , "SPI TX")
    eventDefine(EVENT_SLOT_SPI_RX              , (0x02a LH LE)         , "SPI RX")
    eventDefine(EVENT_SLOT_SPI_TIMEOUT         , (0x02b LH LE)      , "SPI T/O")

    eventDefine(EVENT_SLOT_MESSAGE_SEND        , (0x030 LH LE)      , "Send SPI Req")
    eventDefine(EVENT_SLOT_RESPONSE            , (0x031 LH LE)      , "Get SPI Resp")
    eventDefine(EVENT_SLOT_ERROR               , (0x032 LH LE)      , "SPI Error")
    eventDefine(EVENT_SLOT_SEQUENCE            , (0x033 LH LE)      , "SPI Sequence #")
    eventDefine(EVENT_SLOT_PARAMETER           , (0x034 LH LE)      , "SPI Value")

    eventDefine(EVENT_SLOT_EVENT               , (0x038 LH LE)      , "SPI Event")
    eventDefine(EVENT_SLOT_CHAR_TX             , (0x039 LH LE)      , "SPI Char TX")
    eventDefine(EVENT_SLOT_CHAR_RX             , (0x03a LH LE)      , "SPI Char RX")

    eventDefine(EVENT_MEMORY_ALLOCATE          , (0x040 LO L2)      , "Malloc")
    eventDefine(EVENT_MEMORY_FREE              , (0x041 LO)         , "Free")
    eventDefine(EVENT_MEMORY_REALLOCATE        , (0x042 LO L2)      , "Realloc")

    eventDefine(EVENT_SYSCALL_OPEN             , (0x048 LO L2 LE)   , "SYS fopen")
    eventDefine(EVENT_SYSCALL_READ             , (0x049 LO L2 LE)   , "SYS fread")
    eventDefine(EVENT_SYSCALL_WRITE            , (0x04a LO L2 LE)   , "SYS fwrite")
    eventDefine(EVENT_SYSCALL_CLOSE            , (0x04b LO L2 LE)   , "SYS fclose")

    eventDefine(EVENT_FLASH_PROGRAM            , (0x050 LH L2 LE)   , "Flash Program")
    eventDefine(EVENT_FLASH_ERROR              , (0x051 LH LE)      , "Flash Error")
    eventDefine(EVENT_FLASH_FAIL               , (0x052 NAM L2 LE)  , "Flash Error")
    eventDefine(EVENT_FLASH_ACTION             , (0x053 NAM L2)     , "Flash Operation")

    eventDefine(EVENT_DEBUG_SW_BREAK_SET       , (0x060 LH L2)      , "SW Break Set")
    eventDefine(EVENT_DEBUG_SW_BREAK_CLEAR     , (0x061 LH L2)      , "SW Break Clear")
    eventDefine(EVENT_DEBUG_HW_BREAK_SET       , (0x062 LH L2)      , "HW Break Set")
    eventDefine(EVENT_DEBUG_HW_BREAK_CLEAR     , (0x063 LH L2)      , "HW Break Clear")
    eventDefine(EVENT_DEBUG_HW_BREAKPOINT      , (0x064 LH L2)      , "HW Breakpoint")

    eventDefine(EVENT_DEBUG_BREAKPOINT_ERROR   , (0x068 LH NAM L2 LE), "Breakpoint Error")

    eventDefine(EVENT_GDB_REQUEST              , (0x080 LO NAM L2)  , "GDB Req.")
    eventDefine(EVENT_GDB_RESPONSE             , (0x081 LO NAM L2)  , "GDB Resp.")
    eventDefine(EVENT_GDB_CHARACTER            , (0x082 LO)         , "GDB Char.")
    eventDefine(EVENT_GDB_IGNORED_CHAR         , (0x083 LO)         , "NonGDB Char.")

    eventDefine(EVENT_GDB_REG_READ             , (0x084 LO L2)      , "GDB Read Reg")
    eventDefine(EVENT_GDB_REG_WRITE            , (0x085 LO L2)      , "GDB Write Reg")
    eventDefine(EVENT_GDB_MEM_READ             , (0x086 LO L2)      , "GDB Read Mem")
    eventDefine(EVENT_GDB_MEM_WRITE            , (0x087 LO L2)      , "GDB Write Mem")

    eventDefine(EVENT_GDB_BREAKPOINT           , (0x088 LO NAM L2 LE), "GDB Break")
    eventDefine(EVENT_GDB_INTERRUPT            , (0x089 LO NAM L2 LE), "GDB Interrupt")
    eventDefine(EVENT_GDB_SET_THREAD           , (0x08a LO NAM L2 LE), "GDB Set Thread")

    eventDefine(EVENT_GDB_OPERATION            , (0x08e LO NAM L2 LE), "GDB Op")
    eventDefine(EVENT_GDB_ERROR                , (0x08f LO NAM L2 LE), "GDB Error")

    eventDefine(EVENT_TIMER_STARTED            , (0x100 LT)         , "Event Timer Started #")
    eventDefine(EVENT_TIMER_EXPIRED            , (0x101 LT)         , "Event Timer Expired #")
    eventDefine(EVENT_TIMER_RESTARTED          , (0x102 LT)         , "Event Timer Restarted #")
    eventDefine(EVENT_TIMER_STOPPED            , (0x103 LT)         , "Event Timer Stopped #")
    eventDefine(EVENT_WATCHDOG_RESET           , (0x104 LT LE)      , "Reset Caused by Watchdog")
    eventDefine(EVENT_RESET                    , (0x105 LT LE)      , "Normal Reset")
    eventDefine(EVENT_TIMER_UNAVAILABLE        , (0x107 LT LE)      , "Event Timer Unavailable")

                                                            
    eventDefine(EVENT_OS_STACK_OVERFLOW        , (0x109 LO)         , "Stack Overflow")
                                                              
    eventDefine(EVENT_OS_EVENT                 , (0x10a LO)         , "OS Event")
    eventDefine(EVENT_SUSPEND_ACTIVATED        , (0x10b LO)         , "Task Suspended")
    eventDefine(EVENT_SUSPEND_TIMED_OUT        , (0x10c LO)         , "Suspend Timeout")
                                                              
    eventDefine(EVENT_INFORM_APPLICATION       , (0x10f LA)         , "Application Event")
                                                              
    eventDefine(EVENT_INPUT_ACTIVE             , (0x10e LR)         , "Input Active")
                
    eventDefine(EVENT_THREADX_ERROR            , (0x120 LN NAM L2 LE), "ThreadX Error")
    eventDefine(EVENT_THREADX_EVENT            , (0x121 LN NAM L2 LE), "ThreadX Event")
    eventDefine(EVENT_NETX_ERROR               , (0x130 LN NAM L2 LE), "NetX Error")
    eventDefine(EVENT_NETX_EVENT               , (0x131 LN NAM L2 LE), "NetX Event")
                                                            
    eventDefine(EVENT_OS_TIME_SLICE            , (0x180 NAM)       , "Thread Time Slice")
    eventDefine(EVENT_OS_RETURN_STATUS         , (0x181)           , "TX Return Status")
    eventDefine(EVENT_OS_TASK_SUSPEND          , (0x182 NAM)       , "Call Thread Suspend")
    eventDefine(EVENT_OS_TASK_SWITCH_OUT       , (0x183 NAM)       , "Int. Thread Suspend")
    eventDefine(EVENT_OS_TASK_PRIORITY         , (0x184)           , "Thread Priority")
    eventDefine(EVENT_OS_TASK_RESUME           , (0x185 NAM)       , "Call Thread Resume")
    eventDefine(EVENT_OS_TASK_SWITCH_IN        , (0x186 NAM)       , "Int. Thread Resume")
    eventDefine(EVENT_OS_QUEUE_READ            , (0x188)           , "Queue Read")
    eventDefine(EVENT_OS_QUEUE_WRITE           , (0x189)           , "Queue Write")
    eventDefine(EVENT_OS_QUEUE_COUNT           , (0x18a)           , "Queue Enqueued")
    eventDefine(EVENT_OS_QUEUE_FREE            , (0x18b)           , "Queue Free")
    eventDefine(EVENT_OS_QUEUE_SIZE            , (0x18c)           , "Queue Size")
    eventDefine(EVENT_OS_QUEUE_PARAMETER       , (0x18d)           , "Queue Total")
    eventDefine(EVENT_OS_UNKNOWN               , (0x1f0 L2)        , "Unknown OS Event")

    eventDefine(EVENT_PACKET_RX                , (0x200 LC)         , "CAN Packet Queue RX")
    eventDefine(EVENT_PACKET_TX                , (0x201 LC)         , "CAN Packet Queue TX")
    eventDefine(EVENT_PACKET_TX_FORCE          , (0x202 LC)         , "CAN Packet Force TX")
    eventDefine(EVENT_PACKET_RX_NOTIFY         , (0x203 LC)         , "CAN RX Notify")
                                                              
    eventDefine(EVENT_PACKET_PUT               , (0x208 LM)         , "CAN Packet Put")
    eventDefine(EVENT_PACKET_GET               , (0x209 LM)         , "CAN Packet Get")
                                                              
    eventDefine(EVENT_GET_MESSAGE              , (0x300 LM)         , "Packet Received")
    eventDefine(EVENT_GET_FRAGMENT             , (0x301 LM)         , "Fragment Received")
    eventDefine(EVENT_GET_IO_FRAGMENT          , (0x302 LM)         , "I/O Fragment Received")
    eventDefine(EVENT_GET_ACK                  , (0x303 LM)         , "ACK Received")
    eventDefine(EVENT_GET_EXPLICIT             , (0x304 LM)         , "Explicit Message Received")
    eventDefine(EVENT_GET_IO                   , (0x305 LM)         , "I/O Message Received")
    eventDefine(EVENT_GET_POLL                 , (0x306 LM)         , "Poll Message Received")
                                                              
    eventDefine(EVENT_ACK_MESSAGE              , (0x308 LM)         , "Packet ACKed")
    eventDefine(EVENT_SEND_MESSAGE             , (0x309 LM)         , "Message Sent")
    eventDefine(EVENT_SEND_FRAGMENT            , (0x30a LM)         , "Message Fragment Sent")
    eventDefine(EVENT_SEND_IO_FRAGMENT         , (0x30b LM)         , "I/O Fragment Sent")
    eventDefine(EVENT_SEND_EXPLICIT            , (0x30c LM)         , "Explicit Message Sent")
    eventDefine(EVENT_SEND_IO                  , (0x30d LM)         , "I/O Message Sent")
    eventDefine(EVENT_SEND_POLL                , (0x30e LM)         , "Poll Message Sent")
                                                              
    eventDefine(EVENT_CREATE_CNXN              , (0x400 LD)         , "Create Connection")
    eventDefine(EVENT_APPLY_CNXN               , (0x401 LD)         , "Apply Connection")
    eventDefine(EVENT_DELETE_CNXN              , (0x402 LD)         , "Delete Connection")
    eventDefine(EVENT_DEVNET_RESET             , (0x403 LD)         , "Device Reset Requested")
    eventDefine(EVENT_FAILED_CNXN              , (0x404 LD LE)      , "TX Req. on Bad Connection")
                                                              
    eventDefine(EVENT_COS_MESSAGE              , (0x408 LD)         , "Send COS/Cyc Message")
    eventDefine(EVENT_COS_ACK                  , (0x409 LD)         , "COS/Cyc Ack Rec")
    eventDefine(EVENT_COS_EPR_TIMEOUT          , (0x40a LD LE)      , "COS/Cyc EPR Timeout")
    eventDefine(EVENT_COS_ACK_TIMEOUT          , (0x40b LD LE)      , "COS/Cyc ACK Timeout")
    eventDefine(EVENT_COS_INH_TIMEOUT          , (0x40c LD)         , "COS/Cyc Prod.Inh. Timeout")
                                                              
    eventDefine(EVENT_OPEN_LINK                , (0x500 LD)         , "Open Connection")
    eventDefine(EVENT_CLOSE_LINK               , (0x501 LD)         , "Close Connection")
    eventDefine(EVENT_UCMM_REQUEST             , (0x502 LD)         , "UCMM Request")
    eventDefine(EVENT_UCMM_RESPONSE            , (0x503 LD)         , "UCMM Response")
    eventDefine(EVENT_UCMM_OPEN                , (0x504 LD)         , "UCMM Open Connection")
    eventDefine(EVENT_UCMM_CLOSE               , (0x505 LD)         , "UCMM Close Connection")
                                                              
    eventDefine(EVENT_LINK_TIMED_OUT           , (0x600 LD LE)      , "Link Timed Out")
    eventDefine(EVENT_FRAGMENT_TIMED_OUT       , (0x601 LD LE)      , "Fragment Timed Out")
    eventDefine(EVENT_FRAGMENT_TX_FAIL         , (0x602 LD LE)      , "Fragemented Message Fail")
    eventDefine(EVENT_FRAGMENT_DISCARDED       , (0x603 LD LE)      , "Fragement Discarded")
    eventDefine(EVENT_DUPMACID_REQ             , (0x604 LD)         , "DUP MACID Msg Received")
    eventDefine(EVENT_DUPMACID_RESP            , (0x605 LD)         , "DUP MACID Msg Sent")
                                                              
    eventDefine(EVENT_EPR_TIMED_OUT            , (0x609 LD LE)      , "Link EPR Timed Out")
    eventDefine(EVENT_RESPONSE_TIMED_OUT       , (0x60a LD LE)      , "Link request Timed Out")
                                                              
    eventDefine(EVENT_ACK_HANDLER              , (0x800 LD)         , "ACK handler code")
    eventDefine(EVENT_IO_IDLE                  , (0x801 LA)         , "I/O Idle Status Received")
    eventDefine(EVENT_PACKET_ERROR             , (0x802 LM LE)      , "Packet processing Error")
    eventDefine(EVENT_UPD_POLL                 , (0x803 LA)         , "Poll Data Updated")

    eventDefine(EVENT_ENET_NO_DEVICE           , (0x840 LN LE)      , "Ethnet Chip not found")
    eventDefine(EVENT_ENET_PHY_MEDIA_CHANGE    , (0x841 LN)         , "PHY Media Status Change")
    eventDefine(EVENT_ENET_HW_PROTOCOL_CHANGE  , (0x842 LN LE)      , "Ethnet Protocol Error")
    eventDefine(EVENT_ENET_CHANGE_STATUS       , (0x843 LN)         , "Ethnet Status Update")
    eventDefine(EVENT_ENET_SET_ADDRESS         , (0x844 LN L2)      , "Set Ethnet Address")
    eventDefine(EVENT_ENET_INIT_FAIL           , (0x845 LN NAM L2)  , "Ethnet OS Init Fail")
    eventDefine(EVENT_ENET_PHY_STATUS          , (0x846 LN LE)      , "Ethnet Phy Status")
    eventDefine(EVENT_ENET_START_INT           , (0x84c LN)         , "Ethnet Interrupt")
    eventDefine(EVENT_ENET_END_INT             , (0x84d LN)         , "Ethnet Interrupt Done")
    eventDefine(EVENT_ENET_RX_INT              , (0x850 LN)         , "Ethnet RX Interrupt")
    eventDefine(EVENT_ENET_RX_NO_PACKET        , (0x851 LN LE)      , "Ethnet No RX Packet")
    eventDefine(EVENT_ENET_RX_NO_BUFFER        , (0x852 LN LE)      , "Ethnet No RX Buffer")
    eventDefine(EVENT_ENET_RX_STATUS           , (0x853 LN)         , "Ethnet RX Status")
    eventDefine(EVENT_ENET_RX_DATA_COPIED      , (0x854 LN L2)      , "Ethnet RX Data Copied")
    eventDefine(EVENT_ENET_RX_ERROR_STATUS     , (0x855 LN LE)      , "Ethnet RX Error")
    eventDefine(EVENT_ENET_RX_OVERRUN          , (0x856 LN LE)      , "Ethnet RX Overrun")
    eventDefine(EVENT_ENET_MMU_INT             , (0x861 LN)         , "Ethnet Alloc Interrupt")
    eventDefine(EVENT_ENET_ALLOC_REQ           , (0x862 LN)         , "Ethnet Alloc Requested")
    eventDefine(EVENT_ENET_ALLOC_NO_DATA       , (0x863 LN LE)      , "Ethnet Alloc No Data")
    eventDefine(EVENT_ENET_ALLOC_FAIL          , (0x864 LN LE)      , "Ethnet Alloc Fail")
    eventDefine(EVENT_ENET_ALLOC_OK            , (0x865 LN)         , "Ethnet Alloc OK")
    eventDefine(EVENT_ENET_TX_INT              , (0x870 LN)         , "Ethnet TX Interrupt")
    eventDefine(EVENT_ENET_TX_STATUS           , (0x871 LN)         , "Ethnet TX Status")
    eventDefine(EVENT_ENET_TX_DATA_COPIED      , (0x872 LN L2)      , "Ethnet TX Data Copied")
    eventDefine(EVENT_ENET_TX_EMPTY            , (0x873 LN)         , "Ethnet TX Empty")
    eventDefine(EVENT_ENET_TX_NO_PACKET        , (0x874 LN LE)      , "Ethnet TX No Data")
    eventDefine(EVENT_ENET_TX_ERROR_STATUS     , (0x875 LN LE)      , "Ethnet TX Error")
    eventDefine(EVENT_ENET_ERROR_COUNTERS      , (0x880 LN LE)      , "Set Ethnet Error Counter")
    eventDefine(EVENT_ENET_MULTICAST_ON        , (0x881 LN L2)      , "Ethnet Add Multicast")
    eventDefine(EVENT_ENET_MULTICAST_OFF       , (0x882 LN L2)      , "Ethnet Clear Multicast")
    eventDefine(EVENT_ENET_NEW_CONFIG          , (0x883 LN)         , "Ethnet New Int. Config")

    eventDefine(EVENT_ENET_SEND_PACKET         , (0x891 LN L2)      , "Ethnet Send Packet")
    eventDefine(EVENT_ENET_RECEIVE_PACKET      , (0x892 LN L2)      , "Ethnet Receive Packet")
    eventDefine(EVENT_ENET_RECEIVE_DATA        , (0x898 LN L2)      , "Ethnet Receive Data")

    eventDefine(EVENT_ENET_RECEIVE_ERROR       , (0x89f LN NAM L2 LE), "Ethnet Receive Error")
    
    eventDefine(EVENT_ENET_DRIVER_INIT         , (0x8a0 LN NAM L2)  , "Ethnet Driver Init.")
    eventDefine(EVENT_ENET_DRIVER_ENABLE       , (0x8a1 LN NAM)     , "Ethnet Driver Enable")
    eventDefine(EVENT_ENET_DRIVER_DISABLE      , (0x8a2 LN NAM)     , "Ethnet Driver Disable")
    eventDefine(EVENT_ENET_DRIVER_SEND         , (0x8a3 LN NAM L2)  , "Ethnet Driver Send")
    eventDefine(EVENT_ENET_DRIVER_BROADCAST    , (0x8a4 LN NAM L2)  , "Ethnet Driver Broadcast")
    eventDefine(EVENT_ENET_DRIVER_ARP_TX       , (0x8a5 LN NAM L2)  , "Ethnet Driver ARP Send")
    eventDefine(EVENT_ENET_DRIVER_RARP_TX      , (0x8a6 LN NAM L2)  , "Ethnet Driver RARP Send")
    eventDefine(EVENT_ENET_DRIVER_IP_RX        , (0x8a7 LN NAM L2)  , "Ethnet Driver IP Receive")
    eventDefine(EVENT_ENET_DRIVER_ARP_RX       , (0x8a8 LN NAM L2)  , "Ethnet Driver ARP Receive")
    eventDefine(EVENT_ENET_DRIVER_RARP_RX      , (0x8a9 LN NAM L2)  , "Ethnet Driver RARP Receive")
    eventDefine(EVENT_ENET_DRIVER_MCAST_EN     , (0x8aa LN NAM L2)  , "Ethnet Driver M/C Enable")
    eventDefine(EVENT_ENET_DRIVER_MCAST_DIS    , (0x8ab LN NAM L2)  , "Ethnet Driver M/C Disable")
    eventDefine(EVENT_ENET_DRIVER_CTC_RX       , (0x8ac LN NAM L2)  , "Ethnet Driver CTCProt RX")
    eventDefine(EVENT_ENET_DRIVER_CTC_TX       , (0x8ad LN NAM L2)  , "Ethnet Driver CTCProt TX")

    eventDefine(EVENT_ENET_DRIVER_BAD_RX       , (0x8ae LN NAM L2)   , "Ethnet Driver Unknown RX")
    eventDefine(EVENT_ENET_DRIVER_BAD_REQUEST  , (0x8af LN NAM L2 LE), "Ethnet Driver Invalid Req.")
    
    eventDefine(EVENT_DHCP_UPDATE              , (0x8b0 LE)          , "DHCP State Updated")
    eventDefine(EVENT_DHCP_IP_UPDATE           , (0x8b1 LE)          , "DHCP IP Updated")
    eventDefine(EVENT_DHCP_SV_UPDATE           , (0x8b2 LE)          , "DHCP Server Updated")
    eventDefine(EVENT_DHCP_LT_UPDATE           , (0x8b3 LE)          , "DHCP Lease Updated")
    eventDefine(EVENT_DHCP_RT_UPDATE           , (0x8b4 LE)          , "DHCP Renew Updated")
    eventDefine(EVENT_DHCP_RBT_UPDATE          , (0x8b5 LE)          , "DHCP Rebind Updated")
    eventDefine(EVENT_DHCP_NM_UPDATE           , (0x8b6 LE)          , "DHCP Net Mask Updated")
    eventDefine(EVENT_DHCP_GW_UPDATE           , (0x8b7 LE)          , "DHCP Gateway Updated")
    eventDefine(EVENT_DHCP_HIP_UPDATE          , (0x8b8 LE)          , "DHCP Force IP Address")
    eventDefine(EVENT_DHCP_HNM_UPDATE          , (0x8b9 LE)          , "DHCP Force IP Mask")
    eventDefine(EVENT_DHCP_TIM_UPDATE          , (0x8ba LE)          , "DHCP Timer Restarted")
                                   
    eventDefine(EVENT_PACKET_CREATE            , (0x510 LQ NAM L2) , "Pkt Pool Init")
    eventDefine(EVENT_PACKET_ALLOCATE          , (0x511 LQ NAM L2) , "Pkt Allocate")
    eventDefine(EVENT_PACKET_RELEASE           , (0x512 LQ NAM L2) , "Pkt Release")
    eventDefine(EVENT_PACKET_TX_RELEASE        , (0x513 LQ NAM L2) , "Pkt TX Release")
    eventDefine(EVENT_PACKET_APPEND            , (0x514 LQ NAM L2) , "Pkt Data Add")
    eventDefine(EVENT_PACKET_DELETE            , (0x51f LQ NAM L2) , "Pkt Pool Del")

    eventDefine(EVENT_INTERFACE_PROCESS        , (0x520 LQ NAM)    , "NIC Process")
    eventDefine(EVENT_PERIODIC_UPDATE          , (0x521 LQ NAM L2) , "Periodic U/D")
    eventDefine(EVENT_FRAGMENT_RX              , (0x522 LQ NAM L2) , "Fragment RX")
    eventDefine(EVENT_ARP_QUEUE                , (0x523 LQ NAM L2) , "ARP U/D")
    eventDefine(EVENT_IP_QUEUE                 , (0x524 LQ NAM L2) , "IP U/D")
    eventDefine(EVENT_ICMP_QUEUE               , (0x525 LQ NAM L2) , "ICMP U/D")

    eventDefine(EVENT_ARP_REQUEST              , (0x528 LQ L2)     , "ARP Request")
    eventDefine(EVENT_ARP_RESPONSE             , (0x529 LQ L2)     , "ARP Response")
    eventDefine(EVENT_ARP_UPDATE               , (0x52a LQ)        , "ARP Entry Update")
    eventDefine(EVENT_ARP_DEFER                , (0x52b LQ NAM L2) , "ARP Defer Pkt")

    eventDefine(EVENT_IP_DEFER                 , (0x530 LQ L2)     , "IP Pkt Defer")
    eventDefine(EVENT_IP_PROCESS               , (0x531 LQ L2)     , "IP Pkt Process")
    eventDefine(EVENT_IP_EXTENDED              , (0x532 LQ L2)     , "IP Pkt Options")
    eventDefine(EVENT_IP_CHECKSUM_FAIL         , (0x533 LQ L2)     , "IP Bad Checksum")
    eventDefine(EVENT_IP_CHECKSUM_OK           , (0x534 LQ L2)     , "IP Checksum OK")
    eventDefine(EVENT_IP_FRAGMENT              , (0x535 LQ L2)     , "IP Fragment Received")
    eventDefine(EVENT_IP_FRAGMENT_KILL         , (0x536 LQ L2)     , "IP Fragment Discarded")
    eventDefine(EVENT_IP_RAW_PACKET_RX         , (0x537 LQ L2)     , "IP Raw Packet Received")
    eventDefine(EVENT_IP_DATAGRAM_SIZE         , (0x538 LQ L2)     , "IP Datagram Size")
    eventDefine(EVENT_IP_TRANSMIT              , (0x539 LQ L2)     , "IP Datagram Send")
    eventDefine(EVENT_IP_TRANSMIT_FAIL         , (0x53a LQ L2)     , "IP Transmit Fail")
    eventDefine(EVENT_IP_GET_PHYSICAL          , (0x53b LQ L2)     , "IP Datagram Phy. Address")
    eventDefine(EVENT_IP_PHYSICAL_FOUND        , (0x53c LQ L2)     , "IP Datagram Phy. Search")
    eventDefine(EVENT_IP_TRANSMIT_DISCARD      , (0x53d LQ NAM L2) , "IP TX Discarded")

    eventDefine(EVENT_IP_TX_DONE               , (0x53f LQ L2)     , "IP Datagram Sent")

    eventDefine(EVENT_IP_ICMP                  , (0x540 LQ L2)     , "ICMP Packet Process")
    eventDefine(EVENT_IP_ICMP_START            , (0x541 LQ L2)     , "ICMP Packet Check")
    eventDefine(EVENT_IP_ICMP_CHECKSUM_FAIL    , (0x542 LQ L2)     , "ICMP Bad Checksum")
    eventDefine(EVENT_IP_ICMP_ECHO_RESPONSE    , (0x543 LQ L2)     , "ICMP Echo Response")
    eventDefine(EVENT_IP_ICMP_ECHO_REQUEST     , (0x544 LQ L2)     , "ICMP Echo Request")
    eventDefine(EVENT_IP_ICMP_ECHO_SENT        , (0x545 LQ L2)     , "ICMP Echo Sent")
    eventDefine(EVENT_IP_ICMP_IGNORED          , (0x54f LQ L2)     , "ICMP Packet Ignored")

    eventDefine(EVENT_IP_IGMP                  , (0x550 LQ L2)     , "IGMP Packet Process")
    eventDefine(EVENT_IP_UDP                   , (0x560 LQ L2)     , "UDP Packet Process")
    eventDefine(EVENT_IP_TCP                   , (0x570 LQ L2)     , "TCP Packet Process")

    eventDefine(EVENT_CTC_MSG                  , (0x570 LQ L2)     , "CTC Packet Process")
    eventDefine(EVENT_CTC_QUEUE                , (0x571 LQ NAM L2) , "CTC Queue")
    eventDefine(EVENT_CTC_REQUEST              , (0x572 LQ L2)     , "CTC Request")
    eventDefine(EVENT_CTC_RESPONSE             , (0x573 LQ L2)     , "CTC Response")

    eventDefine(EVENT_IP_UNKNOWN               , (0x5f0 LQ L2)     , "Unknown IP Packet")
    eventDefine(EVENT_IP_FORWARD               , (0x5f1 LQ L2)     , "IP Packet Forwarded")
    eventDefine(EVENT_IP_DISCARD               , (0x5f2 LQ L2)     , "IP Packet Discarded")

    eventDefine(EVENT_READ_EEPROM              , (0x708 LA)         , "EEPROM Read Requested")
    eventDefine(EVENT_WRITE_EEPROM             , (0x709 LA)         , "EEPROM Write Requested")
    eventDefine(EVENT_BAD_REQUEST              , (0x70f LA LE)      , "Invalid App. Request")

    eventDefine(EVENT_SCAN_EXPLICIT            , (0x800 LU)         , "Send Exp. Req.")
    eventDefine(EVENT_SCAN_RESPONSE            , (0x801 LU)         , "Get Exp. Resp.")
    eventDefine(EVENT_SCAN_STROBE              , (0x802 LU)         , "Send BitStrobe")
    eventDefine(EVENT_SCAN_POLL                , (0x804 LU)         , "Send Master Poll")
    eventDefine(EVENT_SCAN_COSACK              , (0x805 LU)         , "Send ACK")

    eventDefine(EVENT_SCAN_CONFIGURE           , (0x806 LU)         , "Configure device")
                                                     
    eventDefine(EVENT_SCAN_SCANTIME            , (0x808 LU)         , "Scan Timing")
    eventDefine(EVENT_SCAN_TIMEOUT             , (0x809 LU LE)      , "Slave Timeout")

    eventDefine(EVENT_SCAN_OPEN                , (0x80a LU)         , "Slave Link Opened")
    eventDefine(EVENT_SCAN_CLOSE               , (0x80b LU)         , "Slave Link Closed")

    eventDefine(EVENT_SCAN_BAD_MFG             , (0x900 LU LE)      , "Manufacturer Bad")
    eventDefine(EVENT_SCAN_BAD_ID              , (0x901 LU LE)      , "Module Type Bad")
    eventDefine(EVENT_SCAN_BAD_VER             , (0x902 LU LE)      , "Version Bad")
    eventDefine(EVENT_SCAN_BAD_PARAM           , (0x903 LU LE)      , "Parameter Mismatch")
    eventDefine(EVENT_SCAN_BAD_LINK            , (0x904 LU LE)      , "Link Bad")

    eventDefine(EVENT_SCAN_BAD_MAP             , (0x908 LU)         , "I/O Map Bad")
    eventDefine(EVENT_SCAN_HIDDEN_MAP          , (0x909 LU LE)      , "I/O map Overlay")

    eventDefine(EVENT_RESOURCE_RX              , (0xa00 LC)         , "Receive Buffer")
    eventDefine(EVENT_RESOURCE_TX              , (0xa01 LC)         , "Transmit Buffer")
    eventDefine(EVENT_RESOURCE_BLOCK           , (0xa02 LC LE)      , "Queue Full")

    eventDefine(EVENT_CTC_ERROR                , (0xa08 LL LE)      , "CTC Error")

    eventDefine(EVENT_INVALID_ACCELERATION     , (0x702 LP LE)      , "Invalid Acceleration")
    eventDefine(EVENT_INVALID_VELOCITY         , (0x703 LP LE)      , "Invalid Velocity")
                                                              
    eventDefine(EVENT_INVALID_REGTIME          , (0x708 LR LE)      , "Registration time invalid")
    eventDefine(EVENT_INVALID_REGVEL           , (0x709 LR LE)      , "Registration velocity invalid")
    eventDefine(EVENT_INVALID_REGDIST          , (0x70a LR LE)      , "Registration distance invalid")
                                                              
    eventDefine(EVENT_ERROR_LIMITED            , (0x800 LS LE)      , "Servo error limit")
    eventDefine(EVENT_INTEGRAL_LIMITED         , (0x801 LS LE)      , "Servo Integrator Limit")
    eventDefine(EVENT_INTEGRAL_CLEARED         , (0x802 LS LE)      , "Servo Integrator Cleared")
    eventDefine(EVENT_OUTPUT0_LIMITED          , (0x804 LS LE)      , "Servo 0 Output Limited")
    eventDefine(EVENT_OUTPUT1_LIMITED          , (0x805 LS LE)      , "Servo 1 Output Limited")

    eventDefine(EVENT_SERVO_STATUS             , (0x806 LS LP)      , "Servo Status Updated")
                                                             
    eventDefine(EVENT_SPEED_LIMITED            , (0x808 LP)         , "Speed Limit reached")
    eventDefine(EVENT_POS_MOTION_PREVENTED     , (0x809 LP)         , "Positive Motion Prevented")
    eventDefine(EVENT_NEG_MOTION_PREVENTED     , (0x809 LP)         , "Negative Motion Prevented")
    eventDefine(EVENT_INVALID_GAINSET          , (0x80a LE)         , "Invalid Servo Gain set")
                                                              
    eventDefine(EVENT_NO_TARGET_SLOT           , (0x900 LP LE)      , "Can't Allocate target data")
    eventDefine(EVENT_NO_TRAJECTORY            , (0x901 LP LE)      , "No Trajectory data")
    eventDefine(EVENT_ADD_TARGET_FAIL          , (0x902 LP LE)      , "Failed to Add Target")
    eventDefine(EVENT_BAD_MASTER               , (0x903 LP LE)      , "Invalid master Axis")
    eventDefine(EVENT_TARGET_MGMT              , (0x904 LP)         , "Target pool management")

    eventDefine(EVENT_UNREACHABLE_DEST         , (0x907 LP LE)      , "Can't reach destination")
                                                              
    eventDefine(EVENT_SEGMENT_ENDED            , (0x908 LP)         , "Finishing segment")
    eventDefine(EVENT_MOTION_ENDED             , (0x909 LP)         , "Zero speed, stopping")
    eventDefine(EVENT_UNENDING_SEGMENT         , (0x90a LP)         , "No segments, continuing")
                                                              
    eventDefine(EVENT_SEGMENT_DONE             , (0x90d LP)         , "Current segment completed")
    eventDefine(EVENT_SEGMENT_SELECTED         , (0x90e LP)         , "Selecting Next Segment")
    eventDefine(EVENT_SEGMENT_STARTED          , (0x90f LP)         , "Starting Segment")
                                                              
    eventDefine(EVENT_AT_POS_ALLOT_FAIL        , (0xa00 LI LE)      , "Can't Allocate PLS data")
    eventDefine(EVENT_AT_POS_FREE_FAIL         , (0xa01 LI LE)      , "Can't Free PLS data")
    eventDefine(EVENT_AT_POS_LINK_FAIL         , (0xa02 LI LE)      , "Can't Link PLS data")
    eventDefine(EVENT_AT_POS_UNLINK_FAIL       , (0xa03 LI LE)      , "Can't Unlink PLS data")
                                                              
    eventDefine(EVENT_AT_POS_CREATE            , (0xa08 LI)         , "Create new PLS event")
    eventDefine(EVENT_AT_POS_SELECT            , (0xa09 LI)         , "Select next PLS event")
    eventDefine(EVENT_AT_POS_SEARCH            , (0xa0a LI)         , "Find next PLS event")
    eventDefine(EVENT_AT_POS_EXECUTE           , (0xa0b LI)         , "Execute PLS Event")
                                                              
    eventDefine(EVENT_HS_TIME_STARTED          , (0xb00 LI)         , "Fast Timing Started")
    eventDefine(EVENT_HS_TIME_RECOGNIZED       , (0xb01 LI)         , "Fast Timing Processing")
    eventDefine(EVENT_HS_TIME_PROCESSED        , (0xb02 LI)         , "Fast Timing Done")
                                                              
    eventDefine(EVENT_PLS_ON                   , (0xb08 LI)         , "PLS Output On")
    eventDefine(EVENT_PLS_OFF                  , (0xb09 LI)         , "PLS Output Off")
    eventDefine(EVENT_PLS_OTHER                , (0xb0a LI)         , "PLS Output Operation")
                                                              
    eventDefine(EVENT_INPUT_CHANGES            , (0xc00 LI)         , "Input Changes Detected")
    eventDefine(EVENT_INPUT_ENABLE             , (0xc01 LI)         , "Input enabled")
    eventDefine(EVENT_INPUT_DISABLE            , (0xc02 LI)         , "Input disabled")
                                                              
    eventDefine(EVENT_REGISTER_IN              , (0xc08 LR LP)      , "Registration Input")
    eventDefine(EVENT_REGISTER_TARGET          , (0xc09 LR LP)      , "Registration Retarget")
    eventDefine(EVENT_REGISTER_DONE            , (0xc0a LR LP)      , "Registration Done")
    eventDefine(EVENT_REGISTRATION             , (0xc0b LR LP)      , "Config. Registration")

    eventDefine(EVENT_INPUT_ACTION_FAIL        , (0xc0f LR LE)      , "Input Assignment fail")
                                                              
    eventDefine(EVENT_HARD_STOP                , (0xd00 LA LS)      , "Servo Hard Stop")
    eventDefine(EVENT_SOFT_STOP                , (0xd01 LA LS)      , "Servo Soft Stop")
    eventDefine(EVENT_MOVE_RELATIVE            , (0xd02 LA LS)      , "Servo Relative Move")
    eventDefine(EVENT_MOVE_ABSOLUTE            , (0xd03 LA LS)      , "Servo Absolute Move")
    eventDefine(EVENT_RUN_AT_SPEED             , (0xd04 LA LS)      , "Servo Run Continuously")
                                                              
    eventDefine(EVENT_PROFILE_SERVO            , (0xd08 LA LS)      , "Profile Servo")
    eventDefine(EVENT_SET_MODE                 , (0xd09 LA LS)      , "Set Servo Mode")
    eventDefine(EVENT_SET_DEADBAND             , (0xd0a LA LS)      , "Set Servo Deadband")
                                                              
    eventDefine(EVENT_PROCESS_COMMAND          , (0xe08 LA LS)      , "Process Command")
    eventDefine(EVENT_START_MOTION             , (0xe09 LA LS)      , "Motion Started")
    eventDefine(EVENT_FORCE_POSITION           , (0xe0a LA LS)      , "Force Position")

    eventDefine(EVENT_SET_RATIO                , (0xb00 LA LS)      , "Set Slave ratio")
                                                              
    eventDefine(EVENT_KEEP_ALIVE               , (0xd00 LS)         , "Keep Alive Msg.")
    eventDefine(EVENT_KEEP_ALIVE_TIMEOUT       , (0xd01 LS)         , "Master Idle Timeout")
                                                              
    eventDefine(EVENT_CHANNEL_A_COMMAND        , (0xd08 LE)         , "Channel 0 Dualport Command")
    eventDefine(EVENT_CHANNEL_B_COMMAND        , (0xd09 LE)         , "Channel 1 Dualport Command")
    eventDefine(EVENT_CHANNEL_STATUS           , (0xd0a LE)         , "Channel Status Update")
                                                              
    eventDefine(EVENT_PARAMETER_POSITION       , (0xc00 LE)         , "Position Value")
    eventDefine(EVENT_PARAMETER_VELOCITY       , (0xc01 LE)         , "Velocity Value")
    eventDefine(EVENT_PARAMETER_ACCELERATION   , (0xc02 LE)         , "Acceleration Value")
    eventDefine(EVENT_PARAMETER_TIME           , (0xc03 LE)         , "Time Value")
    eventDefine(EVENT_PARAMETER_COUNT          , (0xc04 LE)         , "Count Value")
    eventDefine(EVENT_PARAMETER_FLAGS          , (0xc05 LE)         , "Flags Value")
    eventDefine(EVENT_PARAMETER_CHAN           , (0xc06 LE)         , "Channel Value")

    eventDefine(EVENT_PARAMETER_TARGET_POS     , (0xc07 LE)         , "Target Position")
                                                              
    eventDefine(EVENT_PARAMETER_CMD_POSITION   , (0xc08 LE)         , "Cmd. Position Value")
    eventDefine(EVENT_PARAMETER_CMD_VELOCITY   , (0xc09 LE)         , "Cmd. Velocity Value")
    eventDefine(EVENT_PARAMETER_MASTER_POSITION, (0xc0a LE)         , "Master Position Value")
    eventDefine(EVENT_PARAMETER_MASTER_VELOCITY, (0xc0b LE)         , "Master Velocity Value")

    eventDefine(EVENT_QS_TASK_CHECK            , (0xd00 LE L2)      , "QS Task Check")
    eventDefine(EVENT_QS_TASK_DONE             , (0xd01 LE L2)      , "QS Task Done")

    eventDefine(EVENT_DEBUG                    , (0xf00 LE)         , "Diagnostic")
    eventDefine(EVENT_DEBUG_2                  , (0xf01 LE NAM L2)  , "Diagnostic")
eventDefEnd

#undef eventDefine//(n, v, s)  n = v,
#undef eventDefStart//  typedef enum {
#undef eventDefEnd//    } EventCode;
#endif // APP_EVENT
/* End of C++ wrapper started above.  */
#ifdef   __cplusplus
        }
#endif
