/* DOCUMENTATION
* MODULE: vTable.h
*
* Copyright 2000, 2002 Control Technology Corporation as an unpublished work.
* All Rights Reserved.
*
* MODULE DESCRIPTION
* Interface module for vTable.c.
*/

#ifndef uTable_H
#define uTable_H
#ifdef   __cplusplus
extern   "C" {
#endif
//#include "QS2_Abstraction.h"
#define QB_HDR_VERSION 101

// Network Function parameters

/****************************************************************
*  struct USERCONNECTION 									    *
*																*
*  This parameter block must be filled out prior to making any  *
*   Network calls.  This block should be maintained for the     *
*   life of the connection and treated as a connection HANDLE   *
****************************************************************/
typedef struct
{
	int type;		// type of connection... NETWORK_TYPE_UDP...
	int state;		// State of connection
	unsigned char *packetBuf;	// pointer to packet (RX/TX)
	int length;		// number of bytes in Packet buffer (RX/TX)
	int srcPort;		// our port to open socket with, 0 for any
	unsigned long destIp;	// address to connect to or RX is from
	int destPort;		// port to connect to or RX is from
	void (*StateCallback)(void *ptr);  // callback for State change
	unsigned long lhSocket;	// Socket information
	unsigned long lhThread;	// RX Thread information
} USERCONNECTION;

// USERCONNECTION type
#define NETWORK_TYPE_UDP 1
#define NETWORK_TYPE_TCP_CLIENT 2	// not implemented
#define NETWORK_TYPE_TCP_SERVER 3	// not implemented

// USERCONNECTION state phases
#define USER_DISCONNECTED 0	// socket closed, disconnected
#define USER_CONNECTED 2	// connection successful, thread in rx mode
#define USER_DATA_AVAILABLE 3	// rx data available
#define USER_CONNECTING 1	// thread spawned awaiting connection, StateCallback will occur
#define USER_DATAQUEUED 4	// TX data transferred to driver
#define USER_FAILED 5		// operation requested failed

#ifndef CTC_MAX_PATH
typedef struct dateinfo	 // note this is also in 5100RTC.c
{
	short day;
	short month;
	short year;
	short hour;
	short min;
	short sec;
	short dayofweek;
} DATEINFO;

typedef DATEINFO FILETIME;

// Maximum filename allowed
#define CTC_MAX_PATH 128

// Struture used to return information when walking a directory using the findfirst(), findnext() functions
typedef struct 
{
    unsigned long dwFileAttributes; 	// eCTCFileType attributes, only knCTCFileDirectory and knCTCFileNormal are used presently
    FILETIME ftCreationTime; 			// date file created
    unsigned long   nFileSizeHigh; 		// not used, set to 0L
    unsigned long    nFileSizeLow; 		// file size
	unsigned long nTotalAvail;			// Amount of space left that is usable
	unsigned long nTotalDeleted;		// Amount of used space that can be recovered
    char    cFileName[ CTC_MAX_PATH ];  // filename only, no path
    void	*handle;					// FIND_DATA handle to call future findnext() functions, initialized by findfirst()
	void	*drive;						// drive to use for next...
} CTC_FIND_DATA; 

// File handle definition, returned by CTC_fopen()
typedef void *CTC_FILE;

#endif
/* Copy to end of file, below to the uTable.h in User Functions *
*																*
*****************************************************************
*  QUICKSTEP OS VIRTUAL FUNCTION TABLE PROTOTYPE DEFINITIONS	*
*																*
*  Virtual function table for access to User functions.			*
*  This table is initialized by user within their main() 		*
*  function, upon initial loading into memory and prior to		*
*  operation													*
****************************************************************/

/********* Register Access ***************/
typedef RETVAL (*REGISTER_GET)( UINT16 RegNum, INT32 *RegVal);
typedef RETVAL (*REGISTER_PUT)( UINT16 RegNum, INT32 RegVal);

/********* Variant Register Access ***************/
typedef RETVAL (*REGISTER_VGET)( TASK *task, UINT16 RegNum, void *RegVal, int type);
typedef RETVAL (*REGISTER_VPUT)( TASK *task, UINT16 RegNum, void *RegVal, int type);

/********* Register Lock Control ********/
typedef void (*REGISTER_LOCK)(UINT16 RegNum);
typedef void (*REGISTER_UNLOCK)(UINT16 RegNum);

/********* Run Script Command **********/
typedef void *(*RUNCMD_ALLOCATE)(int maxbuffer);
typedef void (*RUNCMD_RELEASE)(void *pCmdP);
typedef long (*RUNCMD_RUN)(void *pCmdP, char *cmd);
typedef int (*RUNCMD_GETRESPONSE)(void *pCmdP,char *buf);

/*********** Task Step Control **********/
typedef RETVAL (*TASK_SETBRANCH)(TASK *task, int step);
typedef void (*TASK_STEPLOCK)(TASK *task);
typedef TASK *(*TASK_CREATETASK)(int stepNum);
typedef RETVAL (*TASK_KILLTASK)(TASK *task);
typedef TASK *(*TASK_GETTASKFROMHANDLE)(unsigned int handle);

/********* Communications ****************/
typedef RETVAL (*COMM_SENDMSG)( INDX port, UINT8 *msg, UINT16 size);
typedef RETVAL (*COMM_GENERICCMD)( INDX port, UINT8 *msg, UINT16 *size);
typedef RETVAL (*COMM_GET_RQST)( INDX port, UINT8 **buf, UINT16 *size);
typedef int (*COMM_NETWORKOPEN)(USERCONNECTION *ptr);	
typedef void (*COMM_NETWORKCLOSE)(USERCONNECTION *ptr);	
typedef int (*COMM_NETWORKSEND)(USERCONNECTION *ptr);	

/************ Motion Routines ****************/
typedef	UINT16 (*MOTION_GET_NUMMOTORS)(void);
typedef	RETVAL (*MOTION_GET_ATTRIBUTE)( UINT8 MotorNum, UINT8 Attribute, STDVAL *value);
typedef	RETVAL (*MOTION_PUT_ATTRIBUTE)( UINT8 MotorNum, UINT8 Attribute, STDVAL value);
typedef	RETVAL (*MOTION_READY)( UINT8 MotorNum, STDVAL *value);
typedef RETVAL (*MOTION_SIMPLE_COMMAND)( UINT8 MotorNum, UINT8 Command);

typedef RETVAL (*RPC_MOTION_PARAM_READ)(TASK *task, int axis, int parameter, int type, void *value, int *rpc_result, int peeridx);
typedef RETVAL (*RPC_MOTION_PARAM_WRITE)(TASK *task, int axis, int parameter, int type, void *value, int *rpc_result, int peeridx);
typedef RETVAL (*RPC_MOTION_CLEAR_ALL_MSB)(TASK *task, int axis, int *rpc_result, int peeridx);
typedef RETVAL (*RPC_MOTION_START_MSB)(TASK *task, int axis, int msb_number, int *rpc_result, int peeridx);
typedef RETVAL (*RPC_MOTION_STOP_MSB)(TASK *task, int axis, int msb_number, int *rpc_result, int peeridx);
typedef RETVAL (*RPC_MOTION_STATUS)(TASK *task, int axis, int *status, int *rpc_result, int peeridx);
typedef RETVAL (*RPC_MOTION_LOAD_MSB)(TASK *task, int axis, int msb_number, void* msb, int size, int *rpc_result, int peeridx);

/************ File System Routines ***********/

// Open the desired file to type of access desired and return handle for further access
typedef CTC_FILE (*CTC_FOPEN)(char *filename, char *mode);
// Get file length
typedef unsigned long (*CTC_FILELENGTH)(CTC_FILE apFile);
// Close file openned by CTC_fclose()
typedef int (*CTC_FCLOSE)(CTC_FILE apFile);
// Check for end of file
typedef int (*CTC_FEOF)( CTC_FILE apFile );
// Write to file
typedef unsigned int (*CTC_FWRITE)( void *buffer, unsigned int size, unsigned int count, CTC_FILE apFile );
// Get current file position
typedef long (*CTC_FTELL)(CTC_FILE apFile);
// Flush open file
typedef void (*CTC_FFLUSH)(CTC_FILE apFile);
// Read from file
typedef unsigned int (*CTC_FREAD)(void *apByteBuffer, unsigned int size, unsigned int length, CTC_FILE apFile );
// Get a line terminated with CR LF from an open file
typedef char *(*CTC_FGETS)(char *string, int count,CTC_FILE handle);
// Write a null terminated string to an open file
typedef int (*CTC_FPUTS)( char *buffer, CTC_FILE apFile);
// Seek to specific position in a file.  Origin is ignored, always from start of file
typedef int (*CTC_FSEEK)( CTC_FILE handle, long offset, int origin );
// Return pointer to the name of the openned file contained within a directory block
typedef char *(*CTC_GETFILENAME)(CTC_FILE handle);

// DIRECTORY FUNCTIONS

// Delete a directory
typedef int (*CTC_RMDIR)(char *DirName);
// Get current position in a directory tree
typedef int (*CTC_PWD)(char *DirName);
// Change directories
typedef int (*CTC_CD)(char *DirName);
// Create a new directory
typedef int (*CTC_MKDIR)(char *DirName);
// Format the disk, must have the disk you are formatting as your default path (use CTC_cd to move to the disk, EX:  cd /RAMDISK
typedef int (*CTC_FORMATDISK)(void);
// Get full file name with path information
typedef int (*CTC_GETFULLFILENAME)(CTC_FILE handle, char *name);
// Get Flash disk information at disk level
typedef void (*CTC_VOLUMEGETINFO)(unsigned long *TotalSize, unsigned long *TotalAvail, unsigned long *TotalDeleted);
// Initialize CTC_FIND_DATA structure and get information on first file, similiar to MSDOS findfirst()
typedef int (*CTC_FINDFIRST)( char *lpFileName,CTC_FIND_DATA *lpFindFileData);
// Get CTC_FIND_DATA information on next file in directory tree, similiar to MSDOS findnext()
typedef int (*CTC_FINDNEXT)( CTC_FIND_DATA *lpFindFileData); 
// Terminate the directory tree walk initiated with the CTC_findfirst() call
typedef void (*CTC_FINDCLOSE)(CTC_FIND_DATA *lpFindFileData);
// Delete a file
typedef int (*CTC_REMOVE)(char *filename);
// Rename a file (ram disk only)
typedef int (*CTC_RENAME)(char *fromName,char *toName);
// Rename a directory (ram disk only)
typedef int (*CTC_RENDIR)(char *fromName,char *toName);
// Get information on a file that is not open, by name and path only
typedef int (*CTC_GETFILEINFO)(char *Name, CTC_FIND_DATA *lpFindFileData);

/*********** System Functions ***************/
typedef unsigned long (*GET_SYSTEM_TICS)(void);		
/*********** Base Threading Functions (Advanced users only) ************/
typedef unsigned int (*THREAD_CREATE)( TX_THREAD *thread_ptr,       /* Task Structure */
						/* Task Name */
                        char *name_ptr,
						/* Routine and Parameter to pass */
                        void (*entry_function)(unsigned long), unsigned long entry_input,
                        /* Stack start and length */
                        void *stack_start, unsigned long stack_size,
                        /* Priority and Threshold */
                        unsigned int priority, unsigned int preempt_threshold, 
						/* time slice */
                        unsigned long time_slice, unsigned int auto_start);

typedef unsigned int (*THREAD_SUSPEND)(TX_THREAD *thread_ptr);  
typedef unsigned int (*THREAD_RESUME)(TX_THREAD *thread_ptr);   
typedef unsigned int (*THREAD_SLEEP)(unsigned long tics);	
typedef unsigned int (*MUTEX_CREATE)(TX_MUTEX *mutex_ptr, char *name,unsigned int inherit);  
typedef unsigned int (*MUTEX_GET)(TX_MUTEX *mutex_ptr, unsigned long wait_option);  
typedef unsigned int (*MUTEX_PUT)(TX_MUTEX *mutex_ptr);	
typedef unsigned int (*MUTEX_DELETE)(TX_MUTEX *mutex_ptr);	

/*********** Diagnostics ***************/
typedef RETVAL (*LOGEVENT)(EventCode event, long parameter);
typedef RETVAL (*LOGEVENTPLUS)(EventCode event, long firstParameter, long secondParameter);
typedef int (*PRINTF)(const char *format, ...);
typedef int (*SPRINTF)(char *buffer, const char *format, ...);

/****** CTCONLY:  System Memory allocation to support malloc() and free()  *******/
typedef unsigned int (*BYTE_ALLOCATE)(TX_BYTE_POOL *pool_ptr,
			void **memory_ptr,unsigned long memory_size,
			unsigned long wait_option); 
typedef unsigned int (*BYTE_RELEASE)(void *memory_ptr); 
typedef unsigned int (*BYTE_POOL_CREATE)(TX_BYTE_POOL *pool_ptr,
			char *name_ptr, void *pool_start, 
			unsigned long pool_size);
typedef unsigned int (*BYTE_POOL_DELETE)(TX_BYTE_POOL *pool_ptr);

/****** CTCONLY:  Interrupt Control  *******/
typedef unsigned int (*INTERRUPT_CONTROL)(unsigned int);

/****** Step Atomicity  *******/
typedef RETVAL (*ENTER_STEP_ATOMICITY)();
typedef RETVAL (*EXIT_STEP_ATOMICITY)();

/****** Watchdog timer control  *******/
typedef RETVAL (*FIREWATCHDOG)();

// Resource Filter Function parameters

// Type of resources to filter
#define RESOURCE_NONE 0
#define RESOURCE_ANALOGIN 1
#define RESOURCE_ANALOGOUT 2
#define RESOURCE_MOTION 3
#define RESOURCE_DIGITALIN 4		// NOT SUPPORTED
#define RESOURCE_DIGITALOUT 5		// NOT SUPPORTED
#define RESOURCE_DATATABLE 6
#define RESOURCE_REGISTER 7

// Filter mode and enabled modes
#define RESOURCE_READ 1		// read only
#define RESOURCE_WRITE 2	// write only

/****************************************************************
*  struct resourceInfo - addResourceFilter parameter block      *
*																*
*  This parameter block must be filled out prior to registering *
*  a callback function for a Quickstep resource.  A pointer to  *
*  it is passed to the addResourceFilter() function             *
****************************************************************/
typedef struct resourceInfo {
	int type;		// Type of resource to add filter to, RESOURCE_ANALOGIN, RESOURCE_ANALOGOUT, etc...
	int mode;		// Type of access to invoke filter on when accessed, RESOURCE_READ, RESOURCE_WRITE
	int start;		// Start number of resource (setting a range)
	int end;		// End number of resource, make same as start if only one (setting a range).
} RESOURCE_INFO;

//#ifndef FILTERPARAMS
/****************************************************************
*  struct filterparams - Resource Filter Function callback block*
*																*
*  Registered resource functions are called with a pointer      *
*  to this structure in order to pass information important     *
*  to the callback funciton.  This is typically information     *
*  with regards to the type of access being performed on the    *
*  resource, resource number, type, etc                         *
****************************************************************/
typedef struct filterparams
{
	int type;			// Type of resource calling, RESOURCE_ANALOGIN, RESOURCE_ANALOGOUT, etc...
	int resourceNum;    // Quickstep resource number for this resource, 1 to 65536 for registers, 0 to nn analog, ...
	int mode;			// Type of access being performed, RESOURCE_READ, RESOURCE_WRITE
	int row;			// If RESOURCE_DATATABLE then the row and column values being accessed are passed
	int col;
	void *option;		// Reserved for future expansion
}FILTERPARAMS;
//#endif

/****************************************************************
* FILTER_FUNCTION                               			    *
*																*
* Type definition for Resource Callback function			    *
*																*
* PARAMETERS:													*
* void *handle - handle returned by addUserResourceFilter when  *
*                function was registered                        *
* FILTERPARAMS *params - Access information block to detail     *
*                        what is being done                     *
* STDVAL value - Current value being read or written.           *
* RETVAL *status - pointer to status code that will be returned *
*                  to Quickstep.  Leaving it unchanged will     *
*                  default to SUCCESS.  Use only for defined    *
*                  errors.                                      *
*																*
* RETURNS:														*
* int - new value to return to Quickstep on a read or value to  *
*       write, if no change then return passed "value"          *
*****************************************************************/
typedef int (*FILTER_FUNCTION)(void *handle, FILTERPARAMS *params, STDVAL value, RETVAL *status);

// Virtual Function table prototype for adding and removing a filter/trigger function
typedef void *(*ADD_RESOURCE_FILTER)(RESOURCE_INFO *rsc, FILTER_FUNCTION func);	
typedef RETVAL (*REMOVE_RESOURCE_FILTER)(void *handle);

#ifndef CMDPARSER
/****************************************************************
*  struct cmdParser - Run Script Command Environment Structure  *
*																*
*  This structure is returned by the allocateCommandParser      *
*  function call and may be re-used for running Script commands.*
****************************************************************/
typedef struct cmdParser
{
	char *aszParameters;	// line to process
	char *m_byteArrayData;	// scratch array to use
	void *ftp;				// where to send results
	void *lhFile;			// file handle
	int mode;				// mode of execution
	int scriptIndex;		// which instance are we
	int lineCtr;			// line of execution in script
	long position;			// position of start of next line
	char *labelName;		// used when branch occurs, pointer to branch label want
	void *task;				// typically NULL
} CMDPARSER;
#endif

/****************************************************************
*  struct vtableQS - Quickstep OS Function table				*
*																*
*  Virtual function table for access to Quickstep functions.	*
*  This table is initialized by Quickstep OS at powerup.		*
****************************************************************/
typedef struct vtableQS
{
	REGISTER_GET            regRead;				// read a Quickstep register
	REGISTER_PUT            regWrite;				// write a Quickstep register

	COMM_SENDMSG            commSendMsg;			// Send a string out a serial port
	COMM_GENERICCMD         commGenericCmd;			// Send a command to a serial port
	COMM_GET_RQST           commReadMsg;			// Read a message from a serial port

	ADD_RESOURCE_FILTER     addResourceFilter;		// add a resource filter to the access list
	REMOVE_RESOURCE_FILTER  removeResourceFilter;	// remove a resource filter from the access list

	GET_SYSTEM_TICS         systemTics;				// Returns number of tics since powerup, in ms.

	PRINTF                  printf;					// printf function redirected to UDP debugger output screen
	SPRINTF                 sprintf;				// sprintf function used by Quickstep, protected by mutex
	LOGEVENTPLUS            logEventPlus;			// Log a predetermined value type and value/string to the event log
	LOGEVENT                logEvent;				// Log a low level value into the debug event log
	FIREWATCHDOG			fireWatchdog;			// Reset a watchdog timer, Quickstep calls it continually.
													// If for any reason not return control must be called in
													// less than 86 ms. or reset will occur.

	ENTER_STEP_ATOMICITY    enterStepAtomicity;		// Obtain ownership of step atomicity
	EXIT_STEP_ATOMICITY     exitStepAtomicity;		// Release ownership of step atomicity...

	/***************************************************************/
	// below virtual functions are for advanced use only and not 
	// offered for general support
	/***************************************************************/
	MOTION_GET_NUMMOTORS    Motion_GetNumMotors;	// Get number of motion objects in system
	MOTION_GET_ATTRIBUTE    Motion_Get_Attribute;
	MOTION_PUT_ATTRIBUTE    Motion_Put_Attribute;
	MOTION_READY            Motion_Ready;			// Get state of motion object
	MOTION_SIMPLE_COMMAND   Motion_Simple_Command;

	THREAD_CREATE           thread_create;			// Create a new thread
	THREAD_SUSPEND          thread_suspend;			// Suspend an existing thread
	THREAD_RESUME           thread_resume;			// Allow a suspended thread to resume operation
	THREAD_SLEEP            thread_sleep;			// Sleep for the specified number of tics (1ms/tic)
	MUTEX_CREATE            mutex_create;			// Create a mutex object
	MUTEX_GET               mutex_get;				// get control of the mutex
	MUTEX_PUT               mutex_put;				// give control of the mutex back
	MUTEX_DELETE            mutex_delete;			// Destroy the mutex, giving all memory back

	BYTE_ALLOCATE           tx_byte_allocate;		// NOT TO BE USED EXCEPT FOR INTERNAL ROUTINES
	BYTE_RELEASE            tx_byte_release;		// NOT TO BE USED EXCEPT FOR INTERNAL ROUTINES
	BYTE_POOL_CREATE        tx_byte_pool_create;	// NOT TO BE USED EXCEPT FOR INTERNAL ROUTINES
	BYTE_POOL_DELETE        tx_byte_pool_delete; 	// NOT TO BE USED EXCEPT FOR INTERNAL ROUTINES
	INTERRUPT_CONTROL    	interruptControl;		// NOT TO BE USED EXCEPT FOR INTERNAL ROUTINES

	COMM_NETWORKOPEN		commNetworkOpen;		// Open a network connection/socket spawning RX thread	
	COMM_NETWORKCLOSE		commNetworkClose;		// Close a network connection
	COMM_NETWORKSEND		commNetworkSend;		// Send data on a network connection
/*********** BELOW ONLY COMPATIBLE WITH 5300 CONTROLLER ************/
	REGISTER_VGET           regVRead;				// read a Quickstep register as a Variant
	REGISTER_VPUT           regVWrite;				// write a Quickstep register as a Variant

	REGISTER_LOCK 			regLock;				// Register mutex lock set
	REGISTER_UNLOCK			regUnlock;				// Register mutex unlock

	TASK_SETBRANCH			taskSetBranch;			// Set the step to branch to upon return from 'C' API
	TASK_STEPLOCK			taskStepLock;			// Set the step lock flag to force execution of next step
													// of present task without loosing atomicity
	TASK_CREATETASK			createTask;				// Create a Quickstep Task
	TASK_KILLTASK			killTask;				// Kill a Quickstep Task
	TASK_GETTASKFROMHANDLE	getTaskFromHandle;		// Get TASK * from a TASK handle
	RUNCMD_ALLOCATE			allocateCommandParser;	// Allocate an environment for Script Command Processing
	RUNCMD_RELEASE			releaseCommandParser;	// Free an environment used for Script Command Processing
	RUNCMD_RUN				runCommandParser;		// Run a Script Command
	RUNCMD_GETRESPONSE		getCommandResponse;		// Get a Script Command response, copying it to a local buffer

	// File system calls

	CTC_FOPEN 				CTC_fopen;   // Open the desired file to type of access desired and return handle for further access
	CTC_FILELENGTH 			CTC_filelength;	// Get file length
	CTC_FCLOSE 				CTC_fclose;	// Close file openned by CTC_fclose()
	CTC_FEOF 				CTC_feof;	  // Check for end of file
	CTC_FWRITE 				CTC_fwrite;	  // Write to file
	CTC_FTELL 				CTC_ftell;   // Get current file position
	CTC_FFLUSH 				CTC_fflush;	// Flush open file
	CTC_FREAD 				CTC_fread;  // Read from file
	CTC_FGETS 				CTC_fgets;  // Get a line terminated with CR LF from an open file
	CTC_FPUTS 				CTC_fputs; // Write a null terminated string to an open file
	CTC_FSEEK 				CTC_fseek;  // Seek to specific position in a file.  Origin is ignored, always from start of file
	CTC_GETFILENAME 		CTC_getFileName; // Return pointer to the name of the openned file contained within a directory block

	// DIRECTORY FUNCTIONS

	CTC_RMDIR 				CTC_rmdir; // Delete a directory
	CTC_PWD 				CTC_pwd;  // Get current position in a directory tree
	CTC_CD 					CTC_cd;	// Change directories
	CTC_MKDIR 				CTC_mkdir;   // Create a new directory
	CTC_FORMATDISK 			CTC_formatDisk;	  // Format the disk
	CTC_GETFULLFILENAME 	CTC_getFullFileName;   // Get full file name with path information
	CTC_VOLUMEGETINFO 		CTC_volumeGetInfo;	// Get Flash disk information at disk level
	CTC_FINDFIRST 			CTC_findfirst;  // Initialize CTC_FIND_DATA structure and get information on first file, similiar to MSDOS findfirst()
	CTC_FINDNEXT 			CTC_findnext; 	// Get CTC_FIND_DATA information on next file in directory tree, similiar to MSDOS findnext()
	CTC_FINDCLOSE 			CTC_findclose;	// Terminate the directory tree walk initiated with the CTC_findfirst() call
	CTC_REMOVE				CTC_remove;	  // Delete a file
	CTC_RENDIR				CTC_rendir;	  // Rename a directory (ram disk and SDISK only)
	CTC_RENAME				CTC_rename;   // Rename a file
	CTC_GETFILEINFO			CTC_getFileInfo;	// Get information on a file that is not open, by name and path only

	// Motion functions
	RPC_MOTION_PARAM_READ		RPC_axis_param_read;
	RPC_MOTION_PARAM_WRITE		RPC_axis_param_write;
	RPC_MOTION_CLEAR_ALL_MSB	RPC_clear_all_MSB;
	RPC_MOTION_START_MSB		RPC_axis_start_MSB;
	RPC_MOTION_STOP_MSB			RPC_axis_stop_MSB;
	RPC_MOTION_STATUS			RPC_axis_status;
	RPC_MOTION_LOAD_MSB			RPC_load_MSB;

} VTABLE_QS;

/****************************************************************
*  USER VIRTUAL FUNCTION TABLE PROTOTYPE DEFINITIONS			*
*																*
*  Virtual function table hooks for access by Quickstep OS 		*
*  to call if defined during strategic points of system			*
*  execution.													*
*  This table is initialized by user within their main() 		*
*  function, with their 'C' functions, as needed, else place	*
*  a NULL in the table entry									*		
****************************************************************/
typedef int   (*USER_INITIALIZATION)(void);       	// Called when started, = main
typedef int   (*USER_OPERATION)(void *);          	// Called to invoke a user operation

typedef void  (*TIMER_INTERRUPT_HOOK)();			// invoked every timer tic, don't stay here long!!!
typedef int   (*TASK_LOOP_HOOK)(SYSMODE state);	  	// invoked after all Quickstep tasks have executed, round robin
typedef	int   (*SUPERTASK_HOOK)(SYSMODE state);	  	// invoked after each Quickstep task
typedef	int   (*SERVICELOOP_HOOK)(SYSMODE state);	// invoked after each Quickstep task
typedef void  (*END_ALL_TASKS_HOOK)();		        // invoked if all tasks are told to end.
typedef void  (*USER_SHUTDOWN)();		        	// invoked if about to load another user module on top of this one.
typedef void  (*NEWTASKSTARTED)(TASK *task);		// invoked when a new task is beginning, created.
typedef void  (*TASKENDED)(TASK *task);				// invoked when a task is shutting down
typedef void  (*QS4CONTROL)(int reason, void *info);	// invoked periodically with 'reason' and 'info', reason specific.  Example - PID control
typedef void  *(*QS4OBJECTTABLE)(int *size);			// invoked to access QS4 object table pointer and size
typedef int (*QS4_OBJACCESS)(int write, int objnum, int propnum, int proptype, void *resultVal, void *blockaccess);  // invoked to access QS4 object property value when indirection flag set

/****************************************************************
*  struct vtableUser - User Defined Function table				*
*																*
*  Virtual function table for access to User functions.			*
*  This table is initialized by user within their main() 		*
*  function, upon initial loading into memory and prior to		*
*  operation													*
****************************************************************/
#define VALID_USER_SIGNATURE  0x10adf7L		// Special identifier at start of User table, required
#define USER_VERSION_MASK     0xffff0000L
#define USER_ENTRIES_MASK     0x000000ffL

#define USER_VERSION_REQUIRED 0x00010000L	// Not currently used
#define USER_ENTRIES_REQUIRED 13				// Number of function entries in the vtableUser structure
typedef struct vtableUser
{
  	long                  signature;	   	// Unique value to verify table is correct type
  	long                  version;			// Version control information to ensure compatibility
  	USER_INITIALIZATION   initialize;		// User routine to be called at start, main() function
  	USER_OPERATION        function;			// NOT USED
	TIMER_INTERRUPT_HOOK  timerTic;			// Invoked on every timer tic (1ms) if defined
	TASK_LOOP_HOOK        taskLoop;			// Invoked as a normal Quickstep step if defined
	SUPERTASK_HOOK        superTaskLoop;	// Invoked after each Quickstep step, like a super task
	SERVICELOOP_HOOK      serviceLoopHook;	// Invoked outside the Quickstep main loop when steps not running
	END_ALL_TASKS_HOOK    endAllTasks;		// Invoked if all tasks are being shutdown by a Quickstep Cancel command
	USER_SHUTDOWN		  userShutdown;	// Invoked when a new "C" module is being loaded for cleanup purposes
	NEWTASKSTARTED		  taskStart;	// Invoked when a new task is beginning, created.
	TASKENDED			  taskEnd;		// invoked when a task is shutting down
	QS4CONTROL			  qs4Control;		// invoked periodically, based on specific 'reason' parameter
	QS4OBJECTTABLE		  QS4ObjectTable;	// invoked to access QS4 object table pointer and size.
	QS4_OBJACCESS		  QS4ObjAccess;		// invoked to access QS4 object table property value when indirection set (real-time value)
} VTABLE_USER;
	
// Prototype declaration of function used to retrieve the core Quickstep OS table.
// Quickstep OS invokes this via a software interrupt and returns the value to the
// User 'C' function.
extern const VTABLE_QS *getCoreTable(void);

//extern VTABLE_QS _coreUserFunctions;

#ifdef   __cplusplus
}
#endif

#endif



